import datetime
import json
import os
import time
from typing import Tuple, Union

from galxiai.const import *
from galxiai.clasreq import ClashAPI, APIBase, ProxyNotFound
from concurrent.futures import ThreadPoolExecutor, as_completed


class CashCat(ClashAPI, APIBase):
    """
    use clash api to connect to the working clash api hub
    """

    def __init__(self):
        super().__init__()
        self._ps = []
        self.available = []
        self.sock = "socks5://127.0.0.1:{}"
        self.httpproxy = "http://127.0.0.1:{}"
        self.mixport = 0
        self.work = False
        self.loaded = False
        self.health = 0
        self.vpn_iter_n = 0
        self.error_count = 0
        self._status = 0

    def setup(self):
        self.conf = self.get_conf()
        proxies = self.get_proxies()
        list_proxy = proxies["proxies"]["GLOBAL"]
        print(list_proxy)

    def install_conf(self, configuration: dict) -> "CashCat":
        if 'clash_proxy' not in configuration:
            print("no clash setting in the configuration file.")
            return self

        clash_pro = configuration['clash_proxy']

        if 'secret' in clash_pro:
            print("use secret", clash_pro['secret'])
            self.secret_bearer = clash_pro['secret']

        if 'conn_file' in clash_pro:
            print("read connect path", clash_pro['conn_file'])
            self.read_connection_file(clash_pro['conn_file'])

        if 'selector' in clash_pro:
            print("read selector", clash_pro['selector'])
            CashHub.TEST_CLASH_SELECTOR = clash_pro['selector']

        if 'test_endpoint' in clash_pro:
            print("read test endpoint", clash_pro['test_endpoint'])
            CashHub.TEST_PROTOCOL = clash_pro['test_endpoint']

        return self

    def check_unuse_name(self, connection_name: str):
        for key in CashHub.FILTER:
            if key in connection_name:
                return True
        return False

    def in_thread_scan(self):
        self.available = []
        proxies = self.get_proxies()
        self._ps = proxies["proxies"]["GLOBAL"]["all"]
        for pname in self._ps:
            if self.check_unuse_name(pname):
                continue
            self.available.append({
                "s": pname,
                "delay": 0
            })
        workers = len(self.available)
        with ThreadPoolExecutor(max_workers=workers) as executor:
            future_rs = [executor.submit(self.test_health_proxy, proxy_name['s'], CashHub.TEST_PROTOCOL) for
                         proxy_name in self.available]
            for future in as_completed(future_rs):
                result = future.result()
                if isinstance(result, dict):
                    if result != {}:
                        self.update_delay(result)

    def scan(self):
        self.test_protocol()
        self.conf = self.get_conf()
        self.mixport = self.conf["mixed-port"]
        while True:
            r = datetime.datetime.now()
            self.in_thread_scan()
            print("------", r)
            time.sleep(3)

    def update_delay(self, result: dict):
        updated = False
        for v in self.available:
            if v["s"] == result["s"]:
                v.update({
                    "delay": result["delay"]
                })
                updated = True
                break
        if updated is False:
            return
        base_database = os.path.join(CashHub.CACHE_PATH, "proxy_config.json")
        print("update", result["s"])
        with open(base_database, 'w') as file:
            file.write(json.dumps(self.available, indent=2))
            file.close()

    def rotate_conn(self):
        self.work = False
        while True:
            self._read_connections()
            self.vpn_iter_n += 1
            self._status = 1
            res = self._double_check()
            if res is True or self.work is True:
                break

    @property
    def proxyendpoint(self) -> Union[str, None]:
        if self.work:
            return self.httpproxy.format(self.mixport)
        else:
            return None

    @property
    def now_n(self) -> int:
        return self.vpn_iter_n % len(self.available)

    @property
    def connection(self) -> str:
        return self.available[self.now_n]["s"]

    @property
    def delay(self) -> int:
        return int(self.available[self.now_n]["delay"])

    @delay.setter
    def delay(self, n: int):
        self.available[self.now_n]["delay"] = n

    def _test_current_connect(self):
        try:
            self._status = 1
            _fs = self.test_health_proxy(self.connection, CashHub.TEST_PROTOCOL)
            self.delay = int(_fs["delay"])
            return True
        except Exception as e:
            self.delay = -1
            return False
        # self.update_delay(self.available[self.now_n])

    def _double_check(self):
        if 0 < self.delay < 3000:
            try:
                if self._test_current_connect() and 0 < self.delay < 3000:
                    self.work = self.select_proxy_v1(CashHub.TEST_CLASH_SELECTOR, self.connection)
                    print(f"connected to {self.connection} OK")
                    return True
                else:
                    return False
            except ProxyNotFound:
                print("ProxyNotFound")
                time.sleep(10)
                return False
        else:
            return False



# !/usr/bin/env python
# coding: utf-8
import datetime
import math
import sqlite3
import uuid

from SQLiteAsJSON import ManageDB
from SQLiteAsJSON.SQLiteAsJSON import db_logger
import os.path
import json
import random
from requests import Response
import jwt
from typing import Tuple, Union
from galxiai.const import *
from galxiai.ut import obj_to_tuple, obj_to_string


class DB_Fields:
    member_id: str
    user_addr: str
    has_update_time: bool = False


class MemberMgm(ManageDB, DB_Fields):
    """
    modified blockchain db controller
    """

    def total_count(self, tbl: str) -> int:
        cursor = self.conn.cursor()
        cursor.execute(f"SELECT COUNT(*) as count FROM {tbl}")
        (count,) = cursor.fetchone()
        cursor.close()
        if count > 0:
            print(count)
        return count

    def found_table(self, tableName: str) -> bool:
        sqlStatement = f"SELECT name FROM sqlite_sequence WHERE type='table' AND name='{tableName}'"
        cursor = self.conn.cursor()
        cursor.execute(sqlStatement)
        db_result = cursor.fetchone()
        cursor.close()
        if db_result is None:
            return False
        else:
            return True

    def has_address(self, tbl: str, address_key: str) -> bool:
        cursor = self.conn.cursor()
        cursor.execute(f"SELECT address_evm FROM {tbl} WHERE address_evm = ?", (address_key,))
        db_result = cursor.fetchone()
        cursor.close()
        if db_result is None:
            return False
        else:
            return True

    def has_the_timestamp(self, tbl: str, k: int) -> bool:
        cursor = self.conn.cursor()
        cursor.execute(f"SELECT * FROM {tbl} WHERE reg_time = ?", (k,))
        db_result = cursor.fetchone()
        cursor.close()
        if db_result is None:
            return False
        else:
            return True

    def has_row(self, tbl: str, member_id: int) -> bool:
        cursor = self.conn.cursor()
        cursor.execute(f"SELECT * FROM {tbl} WHERE member_id = ?", (member_id,))
        db_result = cursor.fetchone()
        cursor.close()
        if db_result is None:
            return False
        else:
            return True

    def update_col_unsafe(self, tbl: str, address_key: str, params: dict) -> bool:
        if address_key[0:2] != "0x":
            return False

        try:
            columns = obj_to_string(params)
            # update query
            self.conn.execute(f"UPDATE {tbl} set {columns} WHERE address_evm='{address_key}'")
        except sqlite3.OperationalError:
            return False
        except Exception as E:
            db_logger.error('Data Update Error : ', E)
            return False
        self.conn.commit()
        return True

    def update_col_unsafe_by_member_id(self, tbl: str, member_id: str, params: dict) -> bool:
        try:
            columns = obj_to_string(params)
            # update query
            self.conn.execute(f"UPDATE {tbl} set {columns} WHERE member_id='{member_id}'")
        except sqlite3.OperationalError:
            return False
        except Exception as E:
            db_logger.error('Data Update Error : ', E)
            return False
        self.conn.commit()
        return True

    def insert_simple_row(self, tbl: str, params: dict) -> bool:
        # params["timestamp"] = round(time.time() * 1000)  # Current unix time in milliseconds
        keys, vals = obj_to_tuple(params)
        # insert query
        try:
            query = (
                f'INSERT INTO {tbl} ({keys}) VALUES ({vals})'
            )
            self.conn.execute(query, params)
            self.conn.commit()
        except (
                sqlite3.OperationalError,
                Exception
        ) as e:
            db_logger.error('Data Insert Error : ', e)
            return False
        return True

    def get_member_res(self, tbl: str, address_key: str) -> dict:
        cursor = self.conn.cursor()
        _da = {}
        res_01 = None
        try:
            cursor.execute(f'SELECT member_id, next_action, res FROM {tbl} WHERE address_evm = ?',
                           (address_key,))
            (tx_id, next_d, res_01) = cursor.fetchone()
        except Exception as E:
            db_logger.error('Data Insert Error : ', E)
        cursor.close()
        _da = json.loads(res_01)
        return _da

    def get_member_id(self, tbl: str, address_key: str) -> str:
        cursor = self.conn.cursor()
        _da = {}
        tx_id = ''
        try:
            cursor.execute(f'SELECT member_id, next_action FROM {tbl} WHERE address_evm = ?',
                           (address_key,))
            (tx_id, next_d) = cursor.fetchone()
        except Exception as E:
            db_logger.error('Data Insert Error : ', E)
        cursor.close()
        return tx_id

    def get_ref_code_ids(self, tbl: str, address_key: str) -> Tuple[str, str]:
        cursor = self.conn.cursor()
        _da = {}
        referral_code = ''
        parent_code = ''
        try:
            cursor.execute(f'SELECT member_id,referral_code, parent_code FROM {tbl} WHERE address_evm = ?',
                           (address_key,))
            (tx_id, referral_code, parent_code) = cursor.fetchone()
        except Exception as E:
            db_logger.error('Data Insert Error : ', E)
        cursor.close()
        return referral_code, parent_code

    def get_member_res_from_id(self, tbl: str, user_id: str) -> dict:
        cursor = self.conn.cursor()
        _da = {}
        res_01 = None
        try:
            cursor.execute(f'SELECT member_id, next_action, res FROM {tbl} WHERE member_id = ?',
                           (user_id,))
            (tx_id, next_d, res_01) = cursor.fetchone()
        except Exception as E:
            db_logger.error('Data Insert Error : ', E)
        cursor.close()
        _da = json.loads(res_01)
        return _da

    def get_next_action(self, tbl: str, address_key: str) -> dict:
        cursor = self.conn.cursor()
        _da = {}
        next_action = None
        try:
            cursor.execute(f'SELECT member_id, next_action FROM {tbl} WHERE address_evm = ?',
                           (address_key,))
            (tx_id, next_action) = cursor.fetchone()
            _da = json.loads(next_action)
        except json.JSONDecodeError as e:
            db_logger.error('Its ok now', e)
        except Exception as E:
            db_logger.error('Data Insert Error : ', E)
        cursor.close()
        return _da

    def get_next_action_by_id(self, tbl: str, member_id: str) -> dict:
        cursor = self.conn.cursor()
        _da = {}
        next_action = None
        try:
            cursor.execute(f'SELECT member_id, next_action FROM {tbl} WHERE member_id = ?',
                           (member_id,))
            (tx_id, next_action) = cursor.fetchone()

            _da = json.loads(next_action)
        except json.JSONDecodeError as e:
            db_logger.error('Its ok now', e)
        except Exception as E:
            db_logger.error('Data Insert Error : ', E)
        cursor.close()
        return _da

    def get_raw_json_profile(self, tbl: str, address_key: str) -> dict:
        cursor = self.conn.cursor()
        _da = {}
        profile = None
        try:
            cursor.execute(f'SELECT address_evm, profile FROM {tbl} WHERE address_evm = ?',
                           (address_key,))
            (tx_id, profile) = cursor.fetchone()
            _da = json.loads(profile)
        except Exception as E:
            db_logger.error('Data Insert Error : ', E)
        cursor.close()

        return _da

    def _is_what_ready_res(self, tble_member: str, address: str, key: str, default_flag: bool) -> bool:
        if self.has_address(tble_member, address) is False:
            return False
        from_dat = self.get_next_action(tble_member, address)
        if key in from_dat:
            time_next = from_dat[key]
            return self.get_time_now() > time_next
        print(f"key {key} not exist. decision is OK.")
        return default_flag

    def _is_what_delta(self, tble_member: str, address: str, key: str, default_value: int) -> int:
        if self.has_address(tble_member, address) is False:
            return default_value
        from_dat = self.get_next_action(tble_member, address)
        if key in from_dat:
            time_next = from_dat[key]
            return self.get_time_now() - time_next
        return default_value

    def _is_time_exists(self, tble_member: str, address: str, key: str) -> bool:
        if self.has_address(tble_member, address) is False:
            return False
        from_dat = self.get_next_action(tble_member, address)
        return key in from_dat

    def _check_point_is_zero(self, tble_member: str, address: str, key: str) -> bool:
        # if the email is sent
        if self.has_address(tble_member, address) is False:
            return False
        from_dat = self.get_next_action(tble_member, address)
        if key not in from_dat:
            return False
        value = from_dat[key]
        return value == 0

    def _check_point_recently(self, tble_member: str, address: str, key: str) -> bool:
        # if the email is sent
        if self.has_address(tble_member, address) is False:
            return False
        from_dat = self.get_next_action(tble_member, address)
        if key not in from_dat:
            return False
        value = from_dat[key]
        return value > 0

    def get_time_now(self, future_seconds: int = 0) -> int:
        current_time = datetime.datetime.now().timestamp()
        current_time = int(current_time) + future_seconds
        return current_time

    def get_mem_update_time(self, address: str) -> int:
        cursor = self.conn.cursor()
        cursor.execute(f"SELECT update_time FROM {Galxi.DB_MEMBER} WHERE address_evm = ?", (address,))
        (update_time,) = cursor.fetchone()
        cursor.close()
        return update_time

    def _check_point_update_res(self, tble_member: str, address: str, key: str, next_seconds: int = 3600):
        from_dat = self.get_next_action(tble_member, address)
        from_dat.update({
            key: self.get_time_now(next_seconds)
        })
        self.update_col_unsafe(tble_member, address, {
            "next_action": json.dumps(from_dat)
        })

    def _check_point_zero(self, tble_member: str, address: str, key: str):
        from_dat = self.get_next_action(tble_member, address)
        from_dat.update({
            key: 0
        })
        self.update_col_unsafe(tble_member, address, {
            "next_action": json.dumps(from_dat)
        })

    def get_first_row(self, tbl: str) -> int:
        cursor = self.conn.cursor()
        cursor.execute(f"SELECT member_id FROM {tbl}")
        (member_id,) = cursor.fetchone()
        cursor.close()
        return member_id

    def get_row(self, table: str, address: str):
        cursor = self.conn.cursor()
        cursor.execute(f"SELECT * FROM {table} WHERE address_evm = ?", (address,))
        tuple_answers = cursor.fetchone()
        cursor.close()
        return tuple_answers

    def get_child_code(self, tbl: str, atId: int) -> str:
        cursor = self.conn.cursor()
        cursor.execute(f"SELECT member_id, referral_code FROM {tbl} WHERE member_id = ?", (atId,))
        (member_id, refrral_code) = cursor.fetchone()
        cursor.close()
        return refrral_code

    def keepcopy(self, file_path: str, r: Response):
        try:
            with open(file_path, 'wb') as f:
                for chunk in r.iter_content(chunk_size=8192):
                    f.write(chunk)
        except FileNotFoundError:
            with open(file_path, 'a+') as f:
                for chunk in r.iter_content(chunk_size=8192):
                    f.write(chunk)

    def _update_resource_unsafe(self, address: str, res_file: dict):
        return self.update_member_table_unsafe(address, {
            "res": json.dumps(res_file)
        })

    def _update_resource_unsafe_from_id(self, user_id: str, res_file: dict):
        return self.update_col_unsafe_by_member_id(Galxi.DB_MEMBER, user_id, {
            "res": json.dumps(res_file)
        })

    # member session
    def insert_new(self, update_params: dict) -> bool:
        return self.insert_simple_row(Galxi.DB_MEMBER, update_params)

    def update_member_table_unsafe(self, address: str, update_param: dict) -> bool:
        return self.update_col_unsafe(Galxi.DB_MEMBER, address, update_param)

    def has_the_address(self, address: str) -> bool:
        return self.has_address(Galxi.DB_MEMBER, address)

    def has_the_id(self, twid: str) -> bool:
        return self.has_address(Galxi.DB_MEMBER, twid)

    def has_flag(self, key: str) -> bool:
        t = self.get_member_res(Galxi.DB_MEMBER, self.user_addr)
        if key in t:
            if isinstance(t[key], bool):
                return t[key]
            else:
                return False
        return False

    def _is_check_point_exist(self, key: str) -> bool:
        if self.has_address(Galxi.DB_MEMBER, self.user_addr) is False:
            return False
        dat_mem = self.get_next_action(Galxi.DB_MEMBER, self.user_addr)
        if key in dat_mem:
            return True
        return False

    def flag_up(self, key: str):
        da = self.get_member_res(Galxi.DB_MEMBER, self.user_addr)
        da.update({key: True})
        self._update_resource_unsafe(self.user_addr, da)

    def flag_down(self, key: str):
        da = self.get_member_res(Galxi.DB_MEMBER, self.user_addr)
        da.update({key: False})
        self._update_resource_unsafe(self.user_addr, da)

    def _check_point_update(self, key: str, how_long_seconds: int = 3600):
        self._check_point_update_res(Galxi.DB_MEMBER, self.user_addr, key, how_long_seconds)

    def _is_what_ready(self, key: str, df: bool = True) -> bool:
        return self._is_what_ready_res(Galxi.DB_MEMBER, self.user_addr, key, df)

    def _is_what_happen_delta(self, key: str, df: int = 0) -> int:
        return self._is_what_delta(Galxi.DB_MEMBER, self.user_addr, key, df)

    def _is_time_next_exists(self, key: str) -> bool:
        return self._is_time_exists(Galxi.DB_MEMBER, self.user_addr, key)

    def set_invalidate_token(self):
        if self.has_the_address(self.user_addr) is False:
            raise Exception("user is not found from the db")
        self._check_point_update("bearer_token_exp", 0)

    def register_new_evm_address(self, evm_address: str, file: dict) -> bool:
        ...

    def gen_user_id(self):
        return str(uuid.uuid1())

    def try_to_check_jwt_token(self, token):
        decoded_token = jwt.decode(jwt=token, options={"verify_signature": False}, verify=False)
        # Get the expiration timestamp from the decoded token
        expiration_timestamp = decoded_token.get('iat')
        if expiration_timestamp is None:
            return False, 0
        else:
            expiration_delta = expiration_timestamp - self.get_time_now()
            return True, expiration_delta

    def set_jwt_token(self, bearer: str):
        has_address = self.has_the_address(self.user_addr)
        jwtdecode = self.try_to_check_jwt_token(bearer)
        result, delta = jwtdecode
        if has_address:
            self.update_res_kv("bearer_token", bearer)
        else:
            self.register_new_evm_address(self.user_addr, {
                "bearer_token": bearer,
            })
        if result:
            self._check_point_update("bearer_token_exp", delta)
        else:
            self._check_point_update("bearer_token_exp", 3600)

        self.update_col_time()

    def check_bearer_token_expired(self) -> bool:
        return self._is_what_ready("bearer_token_exp")

    def get_access_token(self):
        if self.has_the_address(self.user_addr) is False:
            return ""
        if self.check_bearer_token_expired() is True:
            print("token expired")
            return ""
        d = self.get_member_res(Galxi.DB_MEMBER, self.user_addr)
        if "bearer_token" in d:
            f = str(d["bearer_token"]).replace("\n", "")
            return f
        else:
            return ""

    def get_key_res(self, key: str):
        da = self.get_member_res(Galxi.DB_MEMBER, self.user_addr)
        if key in da:
            return da[key]
        else:
            return ""

    def has_res_key(self, key: str):
        res = self.get_member_res(Galxi.DB_MEMBER, self.user_addr)
        return True if key in res else False

    def res_true(self, key: str) -> bool:
        res = self.get_member_res(Galxi.DB_MEMBER, self.user_addr)
        if key in res:
            res_bool = res[key]
            if isinstance(res_bool, bool):
                return res_bool
            else:
                return False
        else:
            return False

    def update_col_time(self):
        if self.has_update_time is False:
            return
        if self.has_the_address(self.user_addr) is False:
            return
        self.update_col_unsafe(Galxi.DB_MEMBER, self.user_addr, {"update_time": self.get_time_now()})

    def update_res_kv(self, key: str, val):
        da = self.get_member_res(Galxi.DB_MEMBER, self.user_addr)
        da.update({key: val})
        self._update_resource_unsafe(self.user_addr, da)
        self.update_col_time()

    def update_res_set(self, val: dict):
        da = self.get_member_res(Galxi.DB_MEMBER, self.user_addr)
        da.update(val)
        self._update_resource_unsafe(self.user_addr, da)
        self.update_col_time()

    # this is the part for the key value
    def kv_has_key(self, name_key: str):
        tbl = Galxi.DB_KV
        cursor = self.conn.cursor()
        cursor.execute(f"SELECT key_name, value_content, update_time FROM {tbl} WHERE key_name = ?", (name_key,))
        db_result = cursor.fetchone()
        cursor.close()
        if db_result is None:
            return False
        else:
            return True

    def kv_get_value_pair(self, name_key: str):
        # returns the value and the timestamp
        tbl = Galxi.DB_KV
        cursor = self.conn.cursor()
        cursor.execute(f"SELECT value_content, update_time FROM {tbl} WHERE key_name = ?", (name_key,))
        db_result = cursor.fetchone()
        cursor.close()
        return db_result

    def kv_set_pair(self, key: str, value, update_time: int = 0) -> bool:
        tbl = Galxi.DB_KV
        params = {
            "value_content": value,
            "update_time": self.get_time_now()
        }
        if update_time == 0:
            params.update({
                "update_time": self.get_time_now()
            })
        else:
            params.update({
                "update_time": update_time
            })
        if self.kv_has_key(key) is True:
            try:
                columns = obj_to_string(params)
                self.conn.execute(f"UPDATE {tbl} set {columns} WHERE key_name='{key}'")
            except Exception as E:
                db_logger.error('Data Update Error : ', E)
                return False
            self.conn.commit()
            return True
        else:
            params.update({
                "key_name": key
            })
            keys, vals = obj_to_tuple(params)
            # insert query
            try:
                query = (
                    f'INSERT INTO {tbl} ({keys}) VALUES ({vals})'
                )
                self.conn.execute(query, params)
                self.conn.commit()
            except (
                    sqlite3.OperationalError,
                    Exception
            ) as e:
                db_logger.error('Data Insert Error : ', e)
                return False
            return True


class GalaBox(MemberMgm):
    def __init__(self):
        self.has_update_time = True

        path_db = os.path.join(Galxi.CACHE_PATH, 'cache.db')
        schema = os.path.join(Galxi.CACHE_PATH, 'schema.json')
        self.cache_db_path = path_db
        new_db = False if os.path.isfile(path_db) else True
        super().__init__(
            db_name=path_db,
            db_config_file_path=schema,
            same_thread=False
        )

        if new_db:
            self.create_table()
        self.user_addr = ""

    def fix_connection_issue(self):
        self.conn = sqlite3.connect(self.cache_db_path, check_same_thread=False, timeout=30)

    def check_token_claim_ready(self) -> bool:
        return self._is_what_ready("token_claim")

    def check_point_token_claim(self, ms_second: int):
        self._check_point_update("token_claim", ms_second)

    def check_point_synced(self, cred_id: str):
        # + 9 hours after will be ready to do it again
        self._check_point_update(f"cred_{cred_id}", 32400)

    def is_task_just_done(self, cred_id: str, in_range: int = 3600):
        kv = self._is_what_happen_delta(f"cred_{cred_id}", 0)
        if kv == 0:
            return False
        kv = kv + 32400
        kv = int(math.fabs(float(kv)))
        if kv < in_range:
            return True
        else:
            return False

    def check_point_claim(self, cred_id: str):
        # + 30 sec after will be ready for this action
        self._check_point_update(f"cred_claim_{cred_id}", 30)

    def checked_email_mark_time(self):
        self._check_point_update("checked_email_code", 0)

    def checked_email_send_time(self):
        self._check_point_update("sent_email_expire", 3600)

    def reset_email_send_time(self):
        self._check_point_zero(Galxi.DB_MEMBER, self.user_addr, "sent_email_expire")

    def is_email_sent_expired(self):
        return self._is_what_ready("sent_email_expire")

    def is_email_sent(self):
        return self._check_point_recently(Galxi.DB_MEMBER, self.user_addr, "sent_email_expire")

    def is_email_sent_reset(self):
        return self._check_point_is_zero(Galxi.DB_MEMBER, self.user_addr, "sent_email_expire")

    def is_check_point_ready(self, cred_id: str):
        return self._is_what_ready(f"cred_{cred_id}")

    def check_claim_ready(self, cred_it: str):
        return self._is_what_ready(f"cred_claim_{cred_it}")

    def is_campaign_claimed_before(self, camp_id: str):
        return self._is_check_point_exist(camp_id)

    def check_point_campaign_done(self, camp_id: str):
        return self._check_point_update(f"campaign_claimed_{camp_id}", 0)

    def check_point_campaign(self, camp_id: str):
        return self._check_point_update(f"campaign_claimed_{camp_id}", 32400)

    def is_check_point_cp_ready(self, camp_id: str):
        return self._is_what_ready(f"campaign_claimed_{camp_id}", False)

    def register_new_evm_address(self, evm_address: str, file: dict) -> bool:
        p = {
            "member_id": self.gen_user_id(),
            "address_evm": self.user_addr,
            "parent_code": "",
            "referral_code": "",
            "next_action": "{}",
            "update_time": self.get_time_now(),
            "res": json.dumps(file)
        }
        return self.insert_new(p)

    def set_address(self, address_vibe: str):
        self.user_addr = address_vibe

    def is_twitter_account_bind(self) -> bool:
        da = self.get_member_res(Galxi.DB_MEMBER, self.user_addr)
        print(da)
        if "twitter_id" in da:
            twitter_id = da["twitter_id"]
            print(f"twitter account ID is found {twitter_id}")
            if twitter_id is None or twitter_id == "":
                return False
            else:
                return True
        return False

    def get_random_historic_parent_code(self) -> str:
        """
        the goal is to get the random child referral code from the existing table members
        :return:
        """
        try:
            count_members = self.total_count(Galxi.DB_MEMBER)
            if count_members > 10:
                random_number = random.randint(1, count_members)
            else:
                first_id = self.get_first_row(Galxi.DB_MEMBER)
                print(f"only use the first row ID", first_id)
                return self.get_child_code(Galxi.DB_MEMBER, first_id)
            print(f"check number {random_number}")
            if self.has_row(Galxi.DB_MEMBER, random_number) is True:
                return self.get_child_code(Galxi.DB_MEMBER, random_number)
            else:
                return self.get_random_historic_parent_code()

        except RecursionError:
            first_id = self.get_first_row(Galxi.DB_MEMBER)
            print(f"only use the first row ID", first_id)
            return self.get_child_code(Galxi.DB_MEMBER, first_id)

    def log_failed_verifications(self):
        if self.kv_has_key("verification_failure") is False:
            self.kv_set_pair("verification_failure", 1)
        else:
            (value, time_next) = self.kv_get_value_pair("verification_failure")
            self.kv_set_pair("verification_failure", int(value) + 1)


class RemoteDB(MemberMgm):
    # database merger
    def __init__(self, path_db: str):

        schema = os.path.join(Galxi.CACHE_PATH, 'schema.json')
        self.cache_db_path = path_db

        new_db = False if os.path.isfile(path_db) else True

        super().__init__(
            db_name=path_db,
            db_config_file_path=schema,
            same_thread=False
        )
        if new_db:
            print("db is not found. need to init DB")
        else:
            self.conn = sqlite3.connect(path_db, check_same_thread=False, timeout=30)
            print("DB is connected.", path_db)

    def merge_tbl1(self, target_db: MemberMgm):
        total_rows = target_db.total_count(Galxi.DB_MEMBER)
        rows_per_page = 1000
        table_f = f"SELECT * FROM {Galxi.DB_MEMBER} LIMIT ? OFFSET ?"
        for offset in range(0, total_rows, rows_per_page):
            cursor = self.conn.cursor()
            cursor.execute(table_f, (rows_per_page, offset))
            all_pool = cursor.fetchall()
            cursor.close()
            perc = float(offset) / float(total_rows)
            print("batch processing {:.2%}".format(perc))
            g = 0
            if perc < 0.75:
                continue

            for (member_id, address_evm, referral_code, parent_code, res, next_act, update_time) in all_pool:
                if target_db.has_the_address(address_evm) is False:
                    target_db.insert_new({
                        "member_id": member_id,
                        "address_evm": address_evm,
                        "parent_code": parent_code,
                        "referral_code": referral_code,
                        "next_action": next_act,
                        "update_time": update_time,
                        "res": res
                    })
                    print(f"new data record.{address_evm} {g}")
                else:
                    if update_time > target_db.get_mem_update_time(address_evm) is True:
                        target_db.update_member_table_unsafe(address_evm, {
                            "next_action": next_act,
                            "update_time": update_time,
                            "res": res
                        })
                        print("update record.", g, address_evm, end="\r")

                    else:
                        print("skip record.", g, address_evm, end="\r")

                g += 1


class MigrateUpgrade(MemberMgm):
    def __init__(self, file_cache: str):
        self.has_update_time = True
        path_db = os.path.join(Galxi.CACHE_PATH, file_cache)
        schema = os.path.join(Galxi.CACHE_PATH, 'schema.json')
        self.cache_db_path = path_db
        new_db = False if os.path.isfile(path_db) else True
        super().__init__(
            db_name=path_db,
            db_config_file_path=schema,
            same_thread=False
        )
        if new_db:
            self.create_table()
        self.user_addr = ""

    def merge_from_remote_db(self, target_db: MemberMgm, addresses_list: list[str]):
        total_rows = target_db.total_count(Galxi.DB_MEMBER)
        print(f"foreign db has rows:: {total_rows}")
        for address_galxe in addresses_list:
            if target_db.has_the_address(address_galxe) and self.has_the_address(address_galxe) is False:
                j = target_db.get_row(Galxi.DB_MEMBER, address_galxe)
                (id, addr, a, b, ne, res, upt) = j
                p = {
                    "member_id": id[:8],
                    "address_evm": addr,
                    "parent_code": a,
                    "referral_code": b,
                    "next_action": ne,
                    "update_time": upt,
                    "res": res
                }
                self.add_new_row(Galxi.DB_MEMBER, p)
                print(f"action on {address_galxe}", end="\r")
        self.statistic_job()

    def add_new_row(self, tbl: str, params: dict) -> bool:
        keys, vals = obj_to_tuple(params)
        try:
            query = (
                f'INSERT INTO {tbl} ({keys}) VALUES ({vals})'
            )
            self.conn.execute(query, params)
            self.conn.commit()
        except (
                sqlite3.OperationalError,
                Exception
        ) as e:
            db_logger.error('Data Insert Error : ', e)
            return False
        return True

    def fetch_all_from_member(self):
        cursor = self.conn.cursor()
        cursor.execute(f"SELECT address_evm, next_action, res FROM {Galxi.DB_MEMBER}")
        all_members = cursor.fetchall()
        cursor.close()
        return all_members

    def statistic_job_interval(self):
        if self.kv_has_key("stats_job") is False:
            self.kv_set_pair("stats_job", 1)
            return True
        else:
            d = self.kv_get_value_pair("stats_job")
            (value, time_next) = d
            if self.get_time_now() > int(time_next) + 3600 * 5:
                stats = int(value) + 1
                self.kv_set_pair("stats_job", stats)
                return True
            else:
                return False

    def statistic_job(self):
        if self.statistic_job_interval() is False:
            return

        total_dp = self.total_count(Galxi.DB_MEMBER)
        self.kv_set_pair("total_member", int(total_dp))
        all_members = self.fetch_all_from_member()
        # this is the example for the statistic job

        f1 = 0
        f2 = 0
        f3 = 0
        f4 = 0

        for (address, next_act, res) in all_members:
            ...

        self.kv_set_pair("...", f1)
        self.kv_set_pair("...", f2)
        self.kv_set_pair("...", f3)
        self.kv_set_pair("...", f4)



from galxiai.xxcl import CashCat
import os
try:
    import tls_client
except:
    os.system('python -m pip install tls-client')
    import tls_client


class BaseReClient:
    session: tls_client.Session
    proxy_agent: CashCat
    agent_x: str
    cookie_use: bool
    _port_vpn: int
    _retry_failure_connection: int = 0

    def handle_limited_ip(self):
        self._retry_failure_connection += 1
        if self.on_vpn() is False:
            if self._retry_failure_connection > 2:
                self._retry_failure_connection = 0
                self.failure_in_connection_try_proxy()
                return False
            return False
        else:
            if self._retry_failure_connection > 2:
                self.proxy_agent.rotate_conn()
                self._retry_failure_connection = 0
                return True
            return False

    @property
    def at_connection_fails(self):
        return self._retry_failure_connection

    # https://amiunique.org/fingerprint
    def urlreq__init__(self, conf: dict):
        self.agent_x = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:123.0) Gecko/20100101 Firefox/123.0"
        self.reset_client()
        self._retry_failure_connection = 0
        self.cookie_use = False
        use_clash = False
        if use_clash:
            self.proxy_agent = CashCat()
            self.proxy_agent.install_conf(conf)
            self.proxy_agent.test_protocol()

    def failure_in_connection_try_proxy(self):
        print("now try using proxy")
        self.port_vpn = 7890
        self.proxy_agent.setup()

    def reset_client(self):
        self.agent_x = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:123.0) Gecko/20100101 Firefox/123.0"
        self.session = tls_client.Session(
            client_identifier="firefox_120",
            supported_signature_algorithms=[
                "ECDSAWithSHA256",
                "SHA256",
                "SHA384",
                "SHA512",
                "SHA256WithRSAEncryption",
                "SHA384WithRSAEncryption",
                "SHA512WithRSAEncryption",
                "ECDSAWithSHA384"
            ],
            supported_versions=["GREASE", "1.3", "1.2"],
            key_share_curves=["GREASE", "X25519"],
            cert_compression_algo="brotli",
            pseudo_header_order=[
                ":method",
                ":authority",
                ":scheme",
                ":path"
            ],
            connection_flow=15663107,
            header_order=[
                "user-agent",
                "accept",
                "accept-encoding",
                "accept-language",
            ]
        )

    def on_vpn(self) -> bool:
        return self._port_vpn > 0

    @property
    def port_vpn(self) -> int:
        """I'm the 'x' property."""
        return self._port_vpn

    @port_vpn.setter
    def port_vpn(self, value: int):
        self._port_vpn = value
        if value > 0:
            self.session.proxies = {
                'http': f'http://127.0.0.1:{value}',
                'https': f'http://127.0.0.1:{value}'
            }
        else:
            self.session.proxies = None
        self.internal_port_vpn(value)

    def internal_port_vpn(self, k: int):
        pass



import json
import os
from typing import Tuple, Union

from galxiai.const import Galxi


def _save_bytes_c(content, file_name):
    """
    save the bytes info into the file
    :param content:
    :param file_name:
    :return:
    """
    file_object = None
    try:
        file_object = open(file_name, 'wb')
        file_object.truncate(0)
    except FileNotFoundError:
        file_object = open(file_name, 'a+')
    finally:
        # file_object.buffer.write(content)
        file_object.write(content)
        file_object.close()


def json_save(content, tmp_file: str = ""):
    if tmp_file is None or tmp_file == "":
        tmp_file = Galxi.TEMP_JSON

    my_str_as_bytes = None

    if isinstance(content, dict):
        my_str_as_bytes = str.encode(json.dumps(content))

    if isinstance(content, str):
        my_str_as_bytes = str.encode(content)

    if my_str_as_bytes is None:
        my_str_as_bytes = content

    path = os.path.join(Galxi.CACHE_PATH, tmp_file)
    _save_bytes_c(my_str_as_bytes, path)


def obj_to_string(update_config) -> str:
    update_string = ''
    index = 0
    for key, value in update_config.items():
        update_string = update_string + f"{key}='{value}'," if index < len(
            update_config) - 1 else update_string + f"{key}='{value}'"
        index = index + 1

    return update_string


def obj_to_tuple(obj) -> Tuple[str, str]:
    """ Parse JSON object and format it for insert_data method

    Parameters:
        obj (dict): The JSON object that should be formatted

    Returns:
        dict: JSON object with keys and values formatted for insert_data method """

    keys = ''
    values = ''
    for key, value in obj.items():
        keys = f'{keys},{key}' if keys != '' else key
        values = f'{values}, :{key}' if values != '' else f':{key}'

    return keys, values


def check_kv_or_update(bs: dict, _doc: dict, key: str):
    if key in _doc:
        val = _doc[key]

        if key[:3] == "has":
            bs.update({
                key: bool(val)
            })
        else:
            if val is None:
                return

            if isinstance(val, str):
                if val == "":
                    return
                bs.update({
                    key: val
                })

            if isinstance(val, bool):
                bs.update({
                    key: bool(val)
                })

            if isinstance(val, list):
                bs.update({
                    key: val
                })



#     Copyright 2023, --- Kam, mailto:kay.hayen@gmail.com
#
#     Part of "Nuitka", an optimizing Python compiler that is compatible and
#     integrates with CPython, but also works on its own.
#
#     Licensed under the Apache License, Version 2.0 (the "License");
#     you may not use this file except in compliance with the License.
#     You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#     Unless required by applicable law or agreed to in writing, software
#     distributed under the License is distributed on an "AS IS" BASIS,
#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#     See the License for the specific language governing permissions and
#     limitations under the License.
#

import logging

from galxiai.sqlite_galxi import *
from galxiai.ut import *
from galxiai.payload import *

logger1 = logging.getLogger("web3.RequestManager")
logger2 = logging.getLogger("web3.providers.HTTPProvider")
logger3 = logging.getLogger("urllib3.connectionpool")
logger4 = logging.getLogger("paramiko.transport")
logger5 = logging.getLogger("invoke")
logger2.setLevel(logging.ERROR)
logger1.setLevel(logging.ERROR)
logger3.setLevel(logging.ERROR)
logger4.setLevel(logging.ERROR)
logger5.setLevel(logging.ERROR)

__version__ = '0.50.9'


import json
import os.path
import time

import requests
from requests import Response
from urllib3.exceptions import NewConnectionError

TESTS = ['http://localhost:9090', 'http://127.0.0.1:9090', 'http://0.0.0.0:9090', 'http://proxy_service:9090',
         'http://xclash:9090', 'http://proxy_service_provider:9090']


class ClashAPIErr(Exception):
    ...


class ProxyNotFound(ClashAPIErr):
    ...


class TimeoutProxy(ClashAPIErr):
    ...


class ProxyDelay(ClashAPIErr):
    ...


class APIBase:
    bind_log_fn = None

    def __init__(self):
        self.error_count = 0
        self.bind_log_fn = None

    def bind_log(self, dn):
        self.bind_log_fn = dn

    def switch_failure(self):
        self.error_count += 1
        if self.error_count > 10:
            self.error_count = 0
            self.errlog(
                "[CLASHX] Tried 10 vpn access point and it keeps fails. Please double check the vpn health from the provider")

    def errlog(self, msg: str):
        if self.bind_log_fn is not None:
            self.bind_log_fn(msg)
        else:
            print(msg)


class ClashAPI:
    _ps: list
    url: str
    secret_bearer: str
    conf: dict
    available: list
    sock: str
    httpproxy: str
    work: bool
    loaded: bool
    health: int
    _status: int
    # 0: off, 1: rotating, 2: rotate done
    error_count: int
    vpn_iter_n: int
    store_vpn_delays: str

    def __init__(self):
        self.url = ""
        self.secret_bearer = ""
        self.conf = {}

    def read_connection_file(self, file_path: str):
        self.store_vpn_delays = file_path
        self._read_connections()

    def _read_connections(self):
        if os.path.isfile(self.store_vpn_delays) is False:
            print("[CLASHX] connection file is not found")
            return
        io = open(self.store_vpn_delays, 'r')
        try:
            self.available = json.loads(io.read())
            io.close()
        except json.JSONDecodeError:
            print("[CLASHX] decode json file error")

    def test_protocol(self):
        n = 0
        err = 0

        while True:
            try:
                print("[CLASHX] looking up clash X environment port.")
                b = n % len(TESTS)
                self.url = TESTS[b]
                self.conf = self.get_conf()
                print(f"[CLASHX] successfully using {self.url}")
                break

            except requests.exceptions.ConnectionError:
                n += 1
                err += 1
                print(f"[CLASHX][ERROR] {self.url} is not open. Keep scan for the next port.")
                if err > 5:
                    break
                time.sleep(1)
                continue

            except Exception:
                n += 1
                err += 1
                print(f"[CLASHX][ERROR] {self.url} is not open. Keep scan for the next port.")
                if err > 5:
                    break
                time.sleep(1)
                continue

    def _get_base(self, endpoint, method='GET', data=None):
        response = self._wrap_base(endpoint, method, data)
        if response.status_code > 300:
            raise Exception(f'[CLASHX][ERROR] Request failed with status code {response.status_code}')
        else:
            if response.status_code == 200:
                return response.json()
            elif response.status_code == 204:
                return {}
            else:
                return {}

    def _wrap_base(self, endpoint: str, method: str, data=None) -> Response:
        _url = f'{self.url}/{endpoint}'
        __header = {
            'Content-type': 'application/json'
        }
        if self.secret_bearer != '':
            __header.update({
                'Authorization': f'Bearer {self.secret_bearer}'
            })
        if method == 'GET':
            response = requests.get(_url, headers=__header, timeout=6)
        elif method == 'POST':
            response = requests.post(_url, headers=__header, data=json.dumps(data))
        elif method == 'PUT':
            if data is not None:
                response = requests.put(_url, headers=__header, data=json.dumps(data))
            else:
                response = requests.put(_url, headers=__header)
        elif method == 'DELETE':
            response = requests.delete(_url, headers=__header)
        else:
            raise Exception("[CLASHX][ERROR] unsupported method")
        return response

    def _put_base(self, endpoint, data=None):
        response = self._wrap_base(endpoint, 'PUT', data)
        if response.status_code > 300:
            if response.status_code == 503:
                raise Exception(f'An error occurred in the delay test')
            elif response.status_code == 504:
                raise TimeoutProxy()
            elif response.status_code == 404:
                raise ProxyNotFound()
            elif response.status_code == 408:
                raise ProxyDelay()
            elif response.status_code == 400:
                raise Exception(f'[CLASHX][ERROR] Format error')
            else:
                raise Exception(f'[CLASHX][ERROR] Request failed with status code {response.status_code}')
        else:
            if response.status_code == 200:
                return response.json()
            elif response.status_code == 204:
                return {}
            else:
                return {}

    def get_conf(self) -> dict:
        return self._get_base('configs')

    def reload_conf(self) -> dict:
        return self._get_base('configs', 'PUT')

    def get_proxies(self) -> dict:
        return self._get_base('proxies')

    def get_proxy(self, name: str) -> dict:
        return self._get_base(f'proxies/{name}')

    def select_proxy_v1(self, selector_group, connection_name: str) -> bool:
        self._put_base(f'proxies/{selector_group}', {
            "name": connection_name
        })
        return True

    def test_health_proxy(self, name: str, target: str) -> dict:
        try:
            end_path = f'proxies/{name}/delay?url={target}&timeout=4000'
            response = self._wrap_base(end_path, 'GET')

            if response.status_code == 200:
                d = response.json()
                return {
                    "s": name,
                    "delay": d["delay"]
                }

            elif response.status_code == 404:
                ...
            elif response.status_code == 503:
                print(f'[CLASHX][ERROR] An error occurred in the delay test', name)
            elif response.status_code == 504:
                print(f'[CLASHX][ERROR] time out', name)
            elif response.status_code == 404:
                print(f'[CLASHX][ERROR] Proxy not found')
            elif response.status_code == 408:
                print(f'[CLASHX][ERROR] Proxy delay test timeout')
        except requests.exceptions.ReadTimeout:
            ...
        except requests.exceptions.ConnectionError:
            time.sleep(5)

        return {}

    def get_proxies_group(self) -> dict:
        return self._get_base('proxy-groups')

    def get_proxy_group(self, name) -> dict:
        return self._get_base(f'proxy-groups/{name}')

    def create_proxy_group(self, group) -> dict:
        return self._get_base('proxy-groups', 'POST', group)

    def update_proxy_group(self, name, group) -> dict:
        return self._get_base(f'proxy-groups/{name}', 'PUT', group)

    def delete_proxy_group(self, name) -> dict:
        return self._get_base(f'proxy-groups/{name}', 'DELETE')

    def get_rules(self) -> dict:
        return self._get_base('rules')

    def get_rule(self, id: str) -> dict:
        return self._get_base(f'rules/{id}')

    def create_rule(self, rule) -> dict:
        return self._get_base('rules', 'POST', rule)

    def update_rule(self, id: str, rule) -> dict:
        return self._get_base(f'rules/{id}', 'PUT', rule)

    def delete_rule(self, id: str) -> dict:
        return self._get_base(f'rules/{id}', 'DELETE')

    def get_dns(self) -> dict:
        return self._get_base('dns')

    def get_dns_provider(self, name) -> dict:
        return self._get_base(f'dns/{name}/providers')

    def get_dns_providers(self) -> dict:
        return self._get_base('dns/providers')

    def create_dns_provider(self, provider) -> dict:
        return self._get_base('dns/providers', 'POST', provider)

    def update_dns_provider(self, name, provider) -> dict:
        return self._get_base(f'dns/providers/{name}', 'PUT', provider)

    def delete_dns_provider(self, name) -> dict:
        return self._get_base(f'dns/providers/{name}', 'DELETE')



# !/usr/bin/env python
# coding: utf-8
import time

from galxiai import ut
from galxiai.req import *
from galxiai.sqlite_galxi import GalaBox
from galxiai.const import *
import random
import json


class Structure:
    debug: bool
    db: GalaBox
    wallet_erc: any
    tasks: list = []
    email_client: any
    _geetest_mode_ai: any
    configurations: dict
    start_from_index: int

    # solution for email registrations only.
    misc_mode: bool
    serial_passes: list[str]
    pass_file_index: int
    worker_type: int


class GalStep:
    loop_index: int = 0

    def repeat_stepping(self, options: list, at_step: int = -1):
        self.loop_index -= 1
        if at_step == -1:
            t = random.choice(options)
            time.sleep(t)
        else:
            wait = options[at_step]
            print("too many request needs to slow down", wait)
            time.sleep(wait)

    def switch_account(self, x: int):
        ...

    def rack_process(self) -> int:
        ...

    def process_loop(self, loop_at: int):
        ...


class GalxiRunner(UrlReq, Structure, GalStep):

    def get_recap_response(self):
        # need to use your own custom AI captura
        return {}

    def internal_port_vpn(self, k: int):
        # self.capcha_provider.enableProxy(k)
        self.init_ai_module(k)

    def init_ai_module(self, port: int):
        ...

    def check_access_token(self, d):
        if isinstance(d, dict):
            if "signin" in d:
                Galxi.BEARER = d["signin"]
                self.db.set_jwt_token(Galxi.BEARER)

    def action_check_jwt(self):
        # check the current bearer validation
        h = rq_data_check_jwt(self.get_recap_response())
        try:
            galxeQL(self.session, h)
        except InvalidToken:
            self.action_sign_in()

    def get_passport_info(self):
        j = rq_data_passport_info(self.wallet_erc.address)
        d = galxeQL(self.session, j)
        if isinstance(d, dict):
            passport_info = d["addressInfo"]
            passport_id = passport_info["id"]
            passport = passport_info["passport"]
            if passport["status"] == "NOT_ISSUED":
                print("need to issue passport")
            else:
                print(passport)

    def get_passport_camp_info(self):
        j = rq_data_get_campaign_info(self.wallet_erc.address, Galxi.GALXE_CAMPAIGN_PASSPORT)
        d = galxeQL(self.session, j)
        if "campaign" in d:
            if d["campaign"]["status"] != "Active":
                print("passport campaign is closed. maybe")
        else:
            print(d)

    def action_call_add_cred(self, cred_it: str, camp_id: str):
        d = None
        try:
            g = rq_data_add_cred(cred_it, self.wallet_erc.address, self.get_recap_response())
            d = galxeQL(self.session, g)
        except FailedToVerifyCaptcha:
            time.sleep(60)
            self.log_failed_verifications()
            self.action_call_add_cred(cred_it, camp_id)
            return

        if isinstance(d, dict):
            self.db.check_point_synced(cred_it)
            self.db.check_point_claim(cred_it)

    def action_claim_points(self, camp_id: str):
        # if self.db.is_check_point_cp_ready(camp_id) is False:
        #    print("claim campaign check point not ready.")
        #    return
        d = None
        try:
            d = galxeQL(self.session, rq_data_claim_points(camp_id, self.wallet_erc.address, self.get_recap_response()))
        except FailedToVerifyCaptcha:
            time.sleep(60)
            self.log_failed_verifications()
            self.action_claim_points(camp_id)
            return
        print("==== claim progress points")
        if isinstance(d, dict):
            if d["prepareParticipate"]["allow"] is True:
                claim_points = d["prepareParticipate"]["loyaltyPointsTxResp"]["TotalClaimedPoints"]
                print(f"Claim points {claim_points}")
                self.db.check_point_campaign(camp_id)
                self.db.update_res_kv(camp_id, True)
            else:
                reason = str(d["prepareParticipate"]["disallowReason"]).lower()
                if "exceed limit" in reason:
                    self.db.update_res_kv(camp_id, True)
                if "empty reward" in reason:
                    self.db.update_res_kv(camp_id, True)

    def action_claim_pt_once(self, camp_id: str):
        if self.db.get_key_res("hasEmail") is False:
            return
        if self.db.has_res_key(camp_id) is True:
            return
        print("====== claim solana point")
        d = galxeQL(self.session, rq_data_campaign_detail(self.wallet_erc.address, camp_id, True))
        if "campaign" in d:
            if "whitelistInfo" in d["campaign"]:
                info = d["campaign"]["whitelistInfo"]
                if info["claimedLoyaltyPoints"] > 0:
                    self.db.update_res_kv(camp_id, True)
                    print("point claimed")
                else:
                    self.claim_that(camp_id)

    def claim_that(self, camp_id):
        d = galxeQL(self.session, rq_data_claim_points(camp_id, self.wallet_erc.address, self.get_recap_response()))
        if isinstance(d, dict):
            if d["prepareParticipate"]["allow"] is True:
                claim_points = d["prepareParticipate"]["loyaltyPointsTxResp"]["TotalClaimedPoints"]
                print(f"Claim Points {claim_points}")
                self.db.update_res_kv(camp_id, True)
            else:
                reason = str(d["prepareParticipate"]["disallowReason"]).lower()
                print(reason)
                if "exceed limit" in reason:
                    self.db.update_res_kv(camp_id, True)
                if "empty reward" in reason:
                    self.db.update_res_kv(camp_id, True)

        else:
            print("error from claim progress once")

    def action_perform_camp(self, camp_id: str):
        d = galxeQL(self.session, rq_data_campaign_detail(self.wallet_erc.address, camp_id, False))
        call_to_actions = []
        if isinstance(d, dict):
            credential_gp = d["campaign"]["credentialGroups"]
            for h in credential_gp:
                if "credentials" in h:
                    f = h["credentials"]
                    if len(f) > 0:
                        action_call = f[0]
                        do_url = action_call["referenceLink"]
                        cred_it = action_call["id"]
                        sync = action_call["lastSync"]
                        if self.check_perform_action(camp_id, cred_it, do_url, sync):
                            print("do task", cred_it, do_url)
                            call_to_actions.append((
                                cred_it, do_url
                            ))
                            self.action_call_add_cred(cred_it, camp_id)
        self.tasks = call_to_actions

    def check_perform_action(self, camp_id, cred_id, do_url, sync) -> bool:
        return self.db.is_check_point_ready(cred_id) is True or sync == 0

    def action_check_credentials(self):
        if len(self.tasks) == 0:
            return
        for task_mission in self.tasks:
            (cred_id, url) = task_mission
            if self.db.check_claim_ready(cred_id) is False:
                raise WaitForClaimPoint()
            d = galxeQL(self.session, rq_data_sync_cred(cred_id, self.wallet_erc.address))
            if "syncCredentialValue" not in d:
                print("error from not able to receive..")
                continue
            contains = d["syncCredentialValue"]
            if "message" in contains:
                msg = str(contains["message"]).lower()
                if "please click the go button and visit the link first. then try again in" in msg:
                    print("check result too fast..")
                    if self.db.is_task_just_done(cred_id) is True:
                        self.db.check_point_claim(cred_id)
                        raise TryPerformMission()
                else:
                    if msg != "none":
                        print(msg)
            if "value" in contains:
                action_result = contains["value"]["allow"]
                if action_result is True:
                    print(f"Check credit for {cred_id} - completed")
        # self.action_claim_points(Galxi.IO_NET_CAMPAIGN_1)

    def action_get_api_token(self):
        if self.db.has_res_key("api_key") is True:
            return
        _doc = galxeQL(self.session, rq_data_update_access_token(self.wallet_erc.address))
        if "updateAccessToken" in _doc:
            self.db.update_res_kv("api_key", _doc["updateAccessToken"]["accessToken"])

    def action_twitter_action_post_twitter(self, text: str):

        return ""

    def twitter_verification_message(self):
        user_id = self.db.get_key_res("id")
        return """Verifying my Twitter account for my #GalxeID gid:{USER_ID} @Galxe 

    galxe.com/id """.format(USER_ID=user_id)

    def action_twitter_ac_bind(self):
        if self.db.res_true("hasTwitter") is True:
            print("twitter ac bound.")
            return

        print("============= confirm twitter ac")
        url = self.action_twitter_action_post_twitter(self.twitter_verification_message())
        _doc = galxeQL(self.session, rq_data_fill_twitter_ac(self.wallet_erc.address, url))
        if "twitterUserID" in _doc and "twitterUserName" in _doc:
            print(".ok 1")
        _doc = galxeQL(self.session, rq_data_twitter_verify(self.wallet_erc.address, url))
        if "twitterUserID" in _doc and "twitterUserName" in _doc:
            print(".ok 2")

        print("twitter is just ac bound.")
        self.action_get_profile()

    def action_email_confirm(self):
        print("============= confirm email")
        _doc = None

        if self.db.is_email_sent() is False:
            print("no email has been sent")
            return

        if self.db.is_email_sent_reset() is True:
            print("resend email is needed")
            return

        if self.db.is_email_sent_expired() is True:
            print("email has been sent over one hour")
            return
        pend_email = self.db.get_key_res("pending_email")
        mail_id = self.db.get_key_res("temp_email_code")
        if pend_email == "" or mail_id == "":
            print("empty email code or email")
            return
        print(f"found pending email: {pend_email}")
        self.action_sign_in()
        ver_code = self.email_client.loop_z(mail_id, pend_email, "galxe")
        _doc = galxeQL(self.session, rq_data_email_verification(pend_email, self.wallet_erc.address, ver_code))
        if isinstance(_doc, dict):
            if "updateEmail" in _doc:
                self.db.update_res_kv("email_code", ver_code)
                print("bound email address SUCCESS")

    def action_email_binding(self):
        self.action_get_profile()
        _doc = None
        print("============= bind email")
        if self.db.res_true("hasEmail") is False:
            if self.db.is_email_sent() is True and self.db.is_email_sent_expired() is False:
                print("just send out the email now needs to check the email")
                return
            while True:
                (use_email, mail_id) = self.email_client.get_email_address()
                try:
                    _doc = galxeQL(self.session, rq_data_email_registration(
                        use_email, self.wallet_erc.address, self.get_recap_response()))

                except FailedToVerifyCaptcha:
                    print("captcha retry for new_email_code, try again in 60s")
                    time.sleep(60)
                    self.log_failed_verifications()
                    continue

                except RequestErro:
                    print("request error")
                    continue

                if isinstance(_doc, dict) is False:
                    print("result is failure.")
                    continue

                if "sendVerificationCode" in _doc:
                    self.db.update_res_kv("temp_email_code", mail_id)
                    self.db.update_res_kv("pending_email", use_email)
                    self.db.checked_email_send_time()
                    print("confirmation email is sent to", use_email)
                    break
        else:
            print("email is bound")
            self.loop_index = 20

    def get_new_user_nickname(self) -> str:
        return ""

    def action_register_username(self):
        self.action_get_profile()
        if self.db.get_key_res("username") == "":
            _name = ""
            while True:
                _name = self.get_new_user_nickname()
                d = galxeQL(self.session, rq_data_username_check(_name))
                if isinstance(d, dict):
                    if d["usernameExist"] is False:
                        break

            d = galxeQL(self.session, rq_data_create(_name, self.wallet_erc.address))
            if "createNewAccount" in d:
                self.db.update_res_kv("id", d["createNewAccount"])
                self.action_get_profile()

    def action_sign_in(self):
        m, sign = self.wallet_erc.sign_galxe_msg()
        _doc = galxeQL(self.session, rq_data_galxe_sign_in(m, sign, self.wallet_erc.address))
        self.check_access_token(_doc)

    def action_update_token(self):
        _doc = galxeQL(self.session, rq_data_update_access_token(self.wallet_erc.address))
        self.check_access_token(_doc)

    def action_init(self):
        found_address_rec = self.db.has_the_address(self.wallet_erc.address)
        bearer_x = self.db.get_access_token() if found_address_rec is True else ""
        if found_address_rec is False or bearer_x == "":
            self.action_sign_in()
            self.action_register_username()
            _doc = galxeQL(self.session, rq_data_galxe_id(self.wallet_erc.address))
            if _doc["galxeIdExist"] is False:
                print("galxe id is not exist.")
        else:
            Galxi.BEARER = bearer_x
            self.action_check_jwt()
        self.action_register_username()

    def log_failed_verifications(self):
        self.db.log_failed_verifications()

    def profile_res_check(self, _d):
        try:
            _info = _d["addressInfo"]
            data = {
                "id": _info["id"],
            }

            ut.check_kv_or_update(data, _info, "id")
            ut.check_kv_or_update(data, _info, "username")
            ut.check_kv_or_update(data, _info, "address")
            ut.check_kv_or_update(data, _info, "hasEmail")
            ut.check_kv_or_update(data, _info, "solanaAddress")
            ut.check_kv_or_update(data, _info, "seiAddress")
            ut.check_kv_or_update(data, _info, "bitcoinAddress")
            ut.check_kv_or_update(data, _info, "twitterUserID")
            ut.check_kv_or_update(data, _info, "twitterUserName")
            ut.check_kv_or_update(data, _info, "email")
            ut.check_kv_or_update(data, _info, "displayNamePref")
            ut.check_kv_or_update(data, _info, "hasWorldcoin")
            ut.check_kv_or_update(data, _info, "hasWorldcoin")
            self.db.update_res_set(data)
            print("profile updated.")
        except ValueError:
            print(_d)

    def action_get_profile(self):
        d = galxeQL(self.session, rq_data_get_profile(self.wallet_erc.address))
        self.profile_res_check(d)

    def create_new_account(self, user_name: str):
        d = galxeQL(self.session, rq_data_create(self.wallet_erc.address, user_name))
        self.db.update_res_kv("id", d["createNewAccount"])



# !/usr/bin/env python
# coding: utf-8
import datetime
import uuid
from galxiai.const import *
from galxiai.ql import *
import os
import json

from galxiai.reqbase import BaseReClient
from galxiai.ut import json_save

try:
    import tls_client
except:
    os.system('python -m pip install tls-client')
    import tls_client


def rq_data_fill_twitter_ac(address: str, twitter_url: str):
    return {
        "operationName": "checkTwitterAccount",
        "query": CHECK_TWITTER_AC,
        "variables": {
            "input": {
                "address": address,
                "twitterURL": twitter_url
            }
        }
    }


def rq_data_twitter_verify(address: str, twitter_url: str):
    return {
        "operationName": "VerifyTwitterAccount",
        "query": VERIFY_TWITTER_AC,
        "variables": {
            "input": {
                "address": address,
                "twitterURL": twitter_url
            }
        }
    }


def rq_data_check_jwt_cap(_captcha: dict):
    return {
        "operationName": "CheckJWT",
        "query": CHECK_JWT_QL,
        "variables": {
            "captchaInput": _captcha,
        },
    }


def rq_data_address_info(first: int = 30):
    return {
        "query": SPACESINADDRESSINFO,
        "variables": {
            "address": "",
            "listSpaceInput": {"first": first},
            "operationName": "SpacesInAddressInfo",
        }
    }


def rq_data_passport_info(var_address: str):
    return {
        "query": PASSPORTINFO,
        "variables": {
            "address": var_address
        }
    }


def rq_data_check_jwt(_captcha: dict):
    return {
        "query": CHECK_JWT_QL,
        "variables": {
            "captchaInput": _captcha,
            "operationName": "CheckJWT",
        },
    }


def rq_data_campaign_detail(address: str, campaign_id: str, with_address: bool):
    return {
        "query": CAMPAIGNDETAILALL,
        "operationName": "CampaignDetailAll",
        "variables": {
            "address": address.lower(),
            "id": campaign_id,
            "withAddress": with_address
        }
    }


def rq_data_get_profile(address: str):
    return {
        "query": PROFILEQ,
        "operationName": "BasicUserInfo",
        "variables": {
            "address": address
        },
    }


def rq_data_get_campaign_info(address: str, camp_id: str):
    return {
        "query": GET_CAMP_INFO,
        "operationName": "CampaignInfo",
        "variables": {
            "address": address,
            "id": camp_id
        },
    }


def rq_data_issue_persona_kyc_id(address: str, signature: str):
    return {
        "query": CREATE_PERSONA_ID,
        "operationName": "GetOrCreateInquiryByAddress",
        "variables": {
            "input": {
                "address": address,
                "signature": signature
            }
        },
    }


def rq_data_get_plus_expire(address: str):
    return {
        "query": PROFILEQ,
        "operationName": "GetGalaxyPlusExpire",
        "variables": {
            "galxeId": address
        },
    }


def rq_data_username_check(name: str):
    return {
        "query": CHECKUSERNAME,
        "operationName": "IsUsernameExisting",
        "variables": {
            "username": name
        }
    }


def rq_data_create(name: str, address_evm: str):
    return {
        "query": CREATENEWACCOUT,
        "operationName": "CreateNewAccount",
        "variables": {
            "input": {
                "username": name,
                "socialUsername": "",
                "schema": f"EVM:{address_evm}",
            }
        }
    }


def rq_data_space_loyalty_pt(address: str):
    return {
        "query": SPACELOYALTYPT,
        "operationName": "SpaceLoyaltyPoints",
        "variables": {
            "id": "46841",
            "address": address
        },
    }


def rq_data_galxe_sign_in(msg: str, signature: str, address: str):
    return {
        "operationName": "SignIn",
        "variables": {
            "input": {
                "address": address,
                "message": msg,
                "signature": signature,
                "addressType": "EVM",
                "publicKey": ""
            }
        },
        "query": SIGNIN
    }


def rq_data_galxe_id(address_evm: str):
    return {
        "query": GALXEIDEXIST,
        "operationName": "GalxeIDExist",
        "variables": {
            "schema": f"EVM:{address_evm}"
        }
    }


def rq_data_update_access_token(address_evm: str):
    return {
        "query": UPDATEACCESSTOKEN,
        "operationName": "UpdateAccessToken",
        "variables": {
            "input": {
                "address": address_evm
            }
        }
    }


def rq_data_email_registration(_email_address: str, _address: str, _captcha: dict):
    return {
        "query": EMAIL_X,
        "operationName": "SendVerifyCode",
        "variables": {
            "input": {
                "address": _address.lower(),
                "captcha": _captcha,
                "email": _email_address
            }
        }
    }


def rq_data_email_verification(_email: str, _address: str, verification_code: str):
    return {
        "query": EMAIL_UPDATE_VERIF,
        "operationName": "UpdateEmail",
        "variables": {
            "input": {
                "address": _address.lower(),
                "email": _email,
                "verificationCode": verification_code
            }
        }
    }


def rq_data_get_whitelist_sites():
    return {
        "query": GETWHITELISTSITES,
        "operationName": "getWhitelistSites"
    }


def rq_data_recommended_campaign_by_user(recommendByCampaignId: str):
    return {
        "query": RECOMMENDCAMPAIGN,
        "variables": {
            "input": {
                "after": "-1",
                "first": 12,
                "recommendByCampaignId": recommendByCampaignId,
                "recommendByUser": ""
            }
        },
        "operationName": "getMessageStatus"
    }


def rq_data_spaces_by_campaign_user(camp_id: str):
    return {
        "operationName": "RecommendSpacesByCampiagnAndUser",
        "variables": {
            "input": {
                "recommendByUser": "",
                "recommendByCampaignId": camp_id,
                "first": 4,
                "after": "-1"
            },
            "spaceCampaignsInput": {
                "statuses": [
                    "Active"
                ]
            }
        },
        "query": RECOMMENDSPACESBYCAMPIAGNUSER
    }


def rq_data_add_cred(cred_it: str, address: str, captcha: dict):
    return {
        "operationName": "AddTypedCredentialItems",
        "variables": {
            "input": {
                "credId": cred_it,
                "operation": "APPEND",
                "items": [
                    address
                ],
                "captcha": captcha,
            }
        },
        "query": ADDCRED
    }


def rq_data_cred_fill_content(cred_it: str, _address: str, content: str):
    return {
        "operationName": "SyncCredentialValue",
        "variables": {
            "input": {
                "syncOptions": {
                    "credId": cred_it,
                    "address": _address,
                    "survey": {
                        "answers": [
                            content
                        ]
                    }
                }
            }
        },
        "query": SYNCCREDVAL
    }


def rq_data_claim_points(camp_id: str, evm_address: str, _captcha: dict):
    return {
        "operationName": "PrepareParticipate",
        "variables": {
            "input": {
                "signature": "",
                "campaignID": camp_id,
                "address": evm_address,
                "mintCount": 1,
                "captcha": _captcha,
                "chain": "ETHEREUM",
            }
        },
        "query": PREPAREPARTICIPATE
    }


def rq_data_sync_cred(cred_it: str, address: str):
    return {
        "operationName": "SyncCredentialValue",
        "variables": {
            "input": {
                "syncOptions": {
                    "credId": cred_it,
                    "address": address
                }
            }
        },
        "query": SYNC_CRED
    }


def rq_data_campaign_participants(campId: str):
    return {
        "query": CAMPAIGNPARTICIPANTS,
        "variables": {
            "pDownload": False,
            "bDownload": False,
            "isParent": False,
            "id": campId,
            "pfirst": 10,
            "pafter": "-1",
            "wfirst": 10,
            "wafter": "-1"
        },
        "operationName": "campaignParticipants"
    }


def ge_header(html_request: bool = False):
    basic = {
        'Host': Galxi.HOST,
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:123.0) Gecko/20100101 Firefox/123.0'
    }
    if html_request is False:
        basic.update({
            'Content-Type': 'application/json',
            'Sec-Fetch-Site': 'cross-site',
            'Sec-Fetch-Dest': 'empty',
            'Sec-Fetch-Mode': 'cors',
            'Origin': Galxi.ORIGIN,
            'Referer': Galxi.REFERER
        })
    else:
        basic.update({
            'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,*/*;q=0.8',
            'Sec-Fetch-Site': 'none',
            'Sec-Fetch-Dest': 'document',
            'Sec-Fetch-Mode': 'navigate',
            'Upgrade-Insecure-Requests': '1',
        })

    if Galxi.BEARER != "":
        basic.update({
            "authorization": Galxi.BEARER
        })

    basic.update({
        "request-id": str(uuid.uuid1()),
        "TE": "trailers",
    })

    return basic


def post_doc_header():
    h = ge_header(html_request=True)
    h.update({
        "Origin": Galxi.ORIGIN,
        "Content-Type": "application/x-www-form-urlencoded",
        "TE": "trailers",
    })
    return h


def galxeQL(session: tls_client.Session, rq_data: dict):
    p = session.post("https://graphigo.prd.galaxy.eco/query", json=rq_data, headers=ge_header())
    return handle_gl_resource(p)


class UrlReq(BaseReClient):
    def useManualCookie(self):
        self.cookie_use = True


def handle_gl_resource(r):
    plain_text = str(r.text).lower()
    if "not found" in plain_text:
        raise UriRouterNotFound()
    if "generated by cloudfront" in plain_text:
        raise NetworkBusy()
    if "ai server is busy" in plain_text:
        print(plain_text)
        raise TooManyRequest()
    if "no users signed in" in plain_text:
        raise NewUser()
    if r.status_code == 504:
        raise TimeoutError()
    if r.status_code == 422:
        raise RequestErro(
            r.status_code,
            "probably missing input values or some sort, go back and double check the requesting json."
        )
    if r.status_code == 403:
        print(r.status_code)
        print(r.content)
        return False
    if r.status_code == 429:
        print("too many request")
        raise TooManyRequest()
    if r.status_code == 503:
        raise TooManyRequest()
    if r.status_code == 500:
        raise InternalServerErr()
    if r.status_code > 200:
        try:
            f = r.json()
            if "errors" in f:
                msg = str(f["errors"][0]["message"]).lower()
                if "failed to verify recaptcha" in msg:
                    raise FailedToVerifyCaptcha()
                raise RequestErro(r.status_code, msg)
        except json.decoder.JSONDecodeError:
            print("no json")
        raise RequestErro(r.status_code, r.content)

    f = r.json()
    if "errors" in f:
        msg = str(f["errors"][0]["message"]).lower()
        t = datetime.datetime.now()
        msg = f"{msg}, {str(t)}"
        if "failed to verify recaptcha" in msg:
            print(msg)
            raise FailedToVerifyCaptcha()
        elif "internal desc = fail to verify recaptcha" in msg:
            print(msg)
            raise FailedToVerifyCaptcha()
        elif "malformed survey cred value" in msg:
            raise TooManyRequest()
        elif "expired signature" in msg:
            raise ExpiredSignatureErr()
        elif "problem with the service. please try again later" in msg:
            raise TooManyRequest()
        elif "please create galxe id" in msg:
            raise GalxeIdNotCreated()
        elif "wrong code, please try again" in msg:
            raise NewEmailCodeNeeded()
        elif "fail to get verification code" in msg:
            raise NewEmailCodeNeeded()
        elif "invalid jwt token" in msg:
            raise InvalidToken()
        else:
            t = datetime.datetime.now()
            msg = f"{msg}, {str(t)}"
            raise RequestErro(r.status_code, msg)
    if "data" in f:
        json_save(r.content)
        return f["data"]



SPACESINADDRESSINFO = """query SpacesInAddressInfo($address: String!, $listSpaceInput: ListSpaceInput!) {
  addressInfo(address: $address) {
    id
    spaces(input: $listSpaceInput) {
      list {
        ...SpaceBasicFrag
        __typename
      }
      __typename
    }
    __typename
  }
}

fragment SpaceBasicFrag on Space {
  id
  name
  info
  thumbnail
  alias
  links
  isVerified
  status
  followersCount
  activeCampaignCount
  __typename
}
"""
CHECK_TWITTER_AC = """mutation checkTwitterAccount($input: VerifyTwitterAccountInput!) {
  checkTwitterAccount(input: $input) {
    address
    twitterUserID
    twitterUserName
    __typename
  }
}
"""
VERIFY_TWITTER_AC = """mutation VerifyTwitterAccount($input: VerifyTwitterAccountInput!) {
  verifyTwitterAccount(input: $input) {
    address
    twitterUserID
    twitterUserName
    __typename
  }
}
"""
PASSPORTINFO = """query PassportInfo($address: String!) {
  addressInfo(address: $address) {
    id
    passport {
      status
      pendingRedactAt
      id
      __typename
    }
    __typename
  }
}
"""
GET_CAMP_INFO = """query CampaignInfo($id: ID!, $address: String!) {
  campaign(id: $id) {
    ...CampaignDetailFrag
    ...CampaignForCampaignDetail
    space {
      ...SpaceDetail
      isAdmin(address: $address)
      __typename
    }
    isBookmarked(address: $address)
    inWatchList
    claimedLoyaltyPoints(address: $address)
    parentCampaign {
      id
      isSequencial
      __typename
    }
    isSequencial
    childrenCampaigns {
      ...CampaignDetailFrag
      parentCampaign {
        id
        isSequencial
        __typename
      }
      __typename
    }
    __typename
  }
}

fragment CampaignDetailFrag on Campaign {
  id
  ...CampaignMedia
  ...CampaignForgePage
  ...CampaignForCampaignParticipantsBox
  name
  numberID
  type
  inWatchList
  cap
  info
  useCred
  smartbalancePreCheck(mintCount: 1)
  smartbalanceDeposited
  formula
  status
  seoImage
  creator
  tags
  thumbnail
  gasType
  isPrivate
  createdAt
  requirementInfo
  description
  enableWhitelist
  chain
  startTime
  endTime
  requireEmail
  requireUsername
  blacklistCountryCodes
  whitelistRegions
  rewardType
  distributionType
  rewardName
  claimEndTime
  loyaltyPoints
  tokenRewardContract {
    id
    address
    chain
    __typename
  }
  tokenReward {
    userTokenAmount
    tokenAddress
    depositedTokenAmount
    tokenRewardId
    tokenDecimal
    tokenLogo
    tokenSymbol
    __typename
  }
  nftHolderSnapshot {
    holderSnapshotBlock
    __typename
  }
  spaceStation {
    id
    address
    chain
    __typename
  }
  ...WhitelistInfoFrag
  ...WhitelistSubgraphFrag
  gamification {
    ...GamificationDetailFrag
    __typename
  }
  creds {
    id
    name
    type
    credType
    credSource
    referenceLink
    description
    lastUpdate
    lastSync
    syncStatus
    credContractNFTHolder {
      timestamp
      __typename
    }
    chain
    eligible(address: $address, campaignId: $id)
    subgraph {
      endpoint
      query
      expression
      __typename
    }
    dimensionConfig
    value {
      gitcoinPassport {
        score
        lastScoreTimestamp
        __typename
      }
      __typename
    }
    commonInfo {
      participateEndTime
      modificationInfo
      __typename
    }
    __typename
  }
  credentialGroups(address: $address) {
    ...CredentialGroupForAddress
    __typename
  }
  rewardInfo {
    discordRole {
      guildId
      guildName
      roleId
      roleName
      inviteLink
      __typename
    }
    premint {
      startTime
      endTime
      chain
      price
      totalSupply
      contractAddress
      banner
      __typename
    }
    loyaltyPoints {
      points
      __typename
    }
    loyaltyPointsMysteryBox {
      points
      weight
      __typename
    }
    __typename
  }
  participants {
    participantsCount
    bountyWinnersCount
    __typename
  }
  taskConfig(address: $address) {
    participateCondition {
      conditions {
        ...ExpressionEntity
        __typename
      }
      conditionalFormula
      eligible
      __typename
    }
    rewardConfigs {
      id
      conditions {
        ...ExpressionEntity
        __typename
      }
      conditionalFormula
      description
      rewards {
        ...ExpressionReward
        __typename
      }
      eligible
      rewardAttrVals {
        attrName
        attrTitle
        attrVal
        __typename
      }
      __typename
    }
    referralConfig {
      id
      conditions {
        ...ExpressionEntity
        __typename
      }
      conditionalFormula
      description
      rewards {
        ...ExpressionReward
        __typename
      }
      eligible
      rewardAttrVals {
        attrName
        attrTitle
        attrVal
        __typename
      }
      __typename
    }
    __typename
  }
  referralCode(address: $address)
  recurringType
  latestRecurringTime
  nftTemplates {
    id
    image
    treasureBack
    __typename
  }
  __typename
}

fragment CampaignMedia on Campaign {
  thumbnail
  rewardName
  type
  gamification {
    id
    type
    __typename
  }
  __typename
}

fragment CredentialGroupForAddress on CredentialGroup {
  id
  description
  credentials {
    ...CredForAddressWithoutMetadata
    __typename
  }
  conditionRelation
  conditions {
    expression
    eligible
    ...CredentialGroupConditionForVerifyButton
    __typename
  }
  rewards {
    expression
    eligible
    rewardCount
    rewardType
    __typename
  }
  rewardAttrVals {
    attrName
    attrTitle
    attrVal
    __typename
  }
  claimedLoyaltyPoints
  __typename
}

fragment CredForAddressWithoutMetadata on Cred {
  id
  name
  type
  credType
  credSource
  referenceLink
  description
  lastUpdate
  lastSync
  syncStatus
  credContractNFTHolder {
    timestamp
    __typename
  }
  chain
  eligible(address: $address)
  subgraph {
    endpoint
    query
    expression
    __typename
  }
  dimensionConfig
  value {
    gitcoinPassport {
      score
      lastScoreTimestamp
      __typename
    }
    __typename
  }
  __typename
}

fragment CredentialGroupConditionForVerifyButton on CredentialGroupCondition {
  expression
  eligibleAddress
  __typename
}

fragment WhitelistInfoFrag on Campaign {
  id
  whitelistInfo(address: $address) {
    address
    maxCount
    usedCount
    claimedLoyaltyPoints
    currentPeriodClaimedLoyaltyPoints
    currentPeriodMaxLoyaltyPoints
    __typename
  }
  __typename
}

fragment WhitelistSubgraphFrag on Campaign {
  id
  whitelistSubgraph {
    query
    endpoint
    expression
    variable
    __typename
  }
  __typename
}

fragment GamificationDetailFrag on Gamification {
  id
  type
  nfts {
    nft {
      id
      animationURL
      category
      powah
      image
      name
      treasureBack
      nftCore {
        ...NftCoreInfoFrag
        __typename
      }
      traits {
        name
        value
        __typename
      }
      __typename
    }
    __typename
  }
  airdrop {
    name
    contractAddress
    token {
      address
      icon
      symbol
      __typename
    }
    merkleTreeUrl
    addressInfo(address: $address) {
      index
      amount {
        amount
        ether
        __typename
      }
      proofs
      __typename
    }
    __typename
  }
  forgeConfig {
    minNFTCount
    maxNFTCount
    requiredNFTs {
      nft {
        category
        powah
        image
        name
        nftCore {
          capable
          contractAddress
          __typename
        }
        __typename
      }
      count
      __typename
    }
    __typename
  }
  __typename
}

fragment NftCoreInfoFrag on NFTCore {
  id
  capable
  chain
  contractAddress
  name
  symbol
  dao {
    id
    name
    logo
    alias
    __typename
  }
  __typename
}

fragment ExpressionEntity on ExprEntity {
  cred {
    id
    name
    type
    credType
    credSource
    dimensionConfig
    referenceLink
    description
    lastUpdate
    lastSync
    chain
    eligible(address: $address)
    metadata {
      visitLink {
        link
        __typename
      }
      twitter {
        isAuthentic
        __typename
      }
      worldcoin {
        dimensions {
          values {
            value
            __typename
          }
          __typename
        }
        __typename
      }
      __typename
    }
    commonInfo {
      participateEndTime
      modificationInfo
      __typename
    }
    __typename
  }
  attrs {
    attrName
    operatorSymbol
    targetValue
    __typename
  }
  attrFormula
  eligible
  eligibleAddress
  __typename
}

fragment ExpressionReward on ExprReward {
  arithmetics {
    ...ExpressionEntity
    __typename
  }
  arithmeticFormula
  rewardType
  rewardCount
  rewardVal
  __typename
}

fragment CampaignForgePage on Campaign {
  id
  numberID
  chain
  spaceStation {
    address
    __typename
  }
  gamification {
    forgeConfig {
      maxNFTCount
      minNFTCount
      requiredNFTs {
        nft {
          category
          __typename
        }
        __typename
      }
      __typename
    }
    __typename
  }
  __typename
}

fragment CampaignForCampaignParticipantsBox on Campaign {
  ...CampaignForParticipantsDialog
  id
  chain
  space {
    id
    isAdmin(address: $address)
    __typename
  }
  participants {
    participants(first: 10, after: "-1", download: false) {
      list {
        address {
          id
          avatar
          __typename
        }
        __typename
      }
      __typename
    }
    participantsCount
    bountyWinners(first: 10, after: "-1", download: false) {
      list {
        createdTime
        address {
          id
          avatar
          __typename
        }
        __typename
      }
      __typename
    }
    bountyWinnersCount
    __typename
  }
  __typename
}

fragment CampaignForParticipantsDialog on Campaign {
  id
  name
  type
  rewardType
  chain
  nftHolderSnapshot {
    holderSnapshotBlock
    __typename
  }
  space {
    isAdmin(address: $address)
    __typename
  }
  rewardInfo {
    discordRole {
      guildName
      roleName
      __typename
    }
    __typename
  }
  __typename
}

fragment SpaceDetail on Space {
  id
  name
  info
  thumbnail
  alias
  status
  links
  isVerified
  discordGuildID
  followersCount
  nftCores(input: {first: 1}) {
    list {
      id
      marketLink
      __typename
    }
    __typename
  }
  __typename
}

fragment CampaignForCampaignDetail on Campaign {
  ...CampaignForGetImage
  ...CampaignForGetMarketLink
  ...CampaignForSiblingSlide
  ...CampaignForCampaignTools
  ...CampaignForCampaignParticipantsBox
  ...CampaignForCampaignTime
  ...CampaignForClaimInfo
  id
  numberID
  isBookmarked(address: $address)
  inWatchList
  name
  description
  space {
    ...SpaceForCampaignInfoSpace
    status
    __typename
  }
  nftTemplates {
    id
    treasureBack
    __typename
  }
  spaceStation {
    address
    __typename
  }
  __typename
}

fragment CampaignForGetImage on Campaign {
  ...GetImageCommon
  nftTemplates {
    image
    __typename
  }
  __typename
}

fragment GetImageCommon on Campaign {
  ...CampaignForTokenObject
  id
  type
  thumbnail
  __typename
}

fragment CampaignForTokenObject on Campaign {
  tokenReward {
    tokenAddress
    tokenSymbol
    tokenDecimal
    tokenLogo
    __typename
  }
  tokenRewardContract {
    id
    chain
    __typename
  }
  __typename
}

fragment CampaignForGetMarketLink on Campaign {
  space {
    nftCores(input: {first: 1}) {
      list {
        id
        marketLink
        __typename
      }
      __typename
    }
    __typename
  }
  __typename
}

fragment CampaignForSiblingSlide on Campaign {
  id
  space {
    id
    alias
    __typename
  }
  parentCampaign {
    id
    thumbnail
    isSequencial
    childrenCampaigns {
      id
      ...CampaignForGetImage
      ...CampaignForCheckFinish
      __typename
    }
    __typename
  }
  __typename
}

fragment CampaignForCheckFinish on Campaign {
  claimedLoyaltyPoints(address: $address)
  whitelistInfo(address: $address) {
    usedCount
    __typename
  }
  __typename
}

fragment SpaceForCampaignInfoSpace on Space {
  ...SpaceForCampaignSpaceFollow
  thumbnail
  name
  isVerified
  categories
  alias
  __typename
}

fragment SpaceForCampaignSpaceFollow on Space {
  id
  isFollowing
  followersCount
  __typename
}

fragment CampaignForCampaignTools on Campaign {
  id
  numberID
  type
  status
  recurringType
  space {
    isAdmin(address: $address)
    alias
    __typename
  }
  childrenCampaigns {
    id
    __typename
  }
  __typename
}

fragment CampaignForCampaignTime on Campaign {
  startTime
  endTime
  __typename
}

fragment CampaignForCalcCampaigClaimNFT on Campaign {
  ...CampaignForCalcCampaignCanClaim
  type
  whitelistInfo(address: $address) {
    usedCount
    __typename
  }
  credentialGroups(address: $address) {
    rewards {
      rewardType
      rewardCount
      __typename
    }
    __typename
  }
  __typename
}

fragment CampaignForCalcCampaignCanClaim on Campaign {
  distributionType
  cap
  type
  __typename
}

fragment CampaignForCalcCampaigClaimCommon on Campaign {
  type
  whitelistInfo(address: $address) {
    usedCount
    maxCount
    __typename
  }
  credentialGroups(address: $address) {
    rewards {
      rewardType
      rewardCount
      __typename
    }
    __typename
  }
  ...CampaignForCalcCampaignCanClaim
  ...CampaignForIsRaffleParticipateEnded
  __typename
}

fragment CampaignForCalcCampaigClaimToken on Campaign {
  ...CampaignForCalcCampaigClaimCommon
  __typename
}

fragment CampaignForCalcCampaigClaimPoints on Campaign {
  type
  loyaltyPoints
  recurringType
  credentialGroups(address: $address) {
    rewards {
      rewardType
      rewardCount
      __typename
    }
    __typename
  }
  whitelistInfo(address: $address) {
    claimedLoyaltyPoints
    currentPeriodClaimedLoyaltyPoints
    __typename
  }
  __typename
}

fragment CampaignForIsRaffleParticipateEnded on Campaign {
  endTime
  __typename
}

fragment CampaignForClaimInfo on Campaign {
  ...CampaignForCalcCampaigClaimNFT
  ...CampaignForCalcCampaigClaimCommon
  ...CampaignForCalcCampaigClaimToken
  ...CampaignForCalcCampaigClaimPoints
  ...CampaignAsCampaignParticipants
  gasType
  __typename
}

fragment CampaignAsCampaignParticipants on Campaign {
  numNFTMinted
  participants {
    participantsCount
    bountyWinnersCount
    __typename
  }
  __typename
}
"""
CREATE_PERSONA_ID = """mutation GetOrCreateInquiryByAddress($input: GetOrCreateInquiryByAddressInput!) {
  getOrCreateInquiryByAddress(input: $input) {
    status
    vendor
    personaInquiry {
      inquiryID
      sessionToken
      declinedReason
      __typename
    }
    __typename
  }
}
"""

PREPAREPARTICIPATE = """mutation PrepareParticipate($input: PrepareParticipateInput!) {
  prepareParticipate(input: $input) {
    allow
    disallowReason
    signature
    nonce
    mintFuncInfo {
      funcName
      nftCoreAddress
      verifyIDs
      powahs
      cap
      __typename
    }
    extLinkResp {
      success
      data
      error
      __typename
    }
    metaTxResp {
      metaSig2
      autoTaskUrl
      metaSpaceAddr
      forwarderAddr
      metaTxHash
      reqQueueing
      __typename
    }
    solanaTxResp {
      mint
      updateAuthority
      explorerUrl
      signedTx
      verifyID
      __typename
    }
    aptosTxResp {
      signatureExpiredAt
      tokenName
      __typename
    }
    tokenRewardCampaignTxResp {
      signatureExpiredAt
      verifyID
      __typename
    }
    loyaltyPointsTxResp {
      TotalClaimedPoints
      __typename
    }
    flowTxResp {
      Name
      Description
      Thumbnail
      __typename
    }
    __typename
  }
}
"""
SYNCCREDVAL = """mutation SyncCredentialValue($input: SyncCredentialValueInput!) {
  syncCredentialValue(input: $input) {
    value {
      address
      spaceUsers {
        follow
        points
        participations
        __typename
      }
      campaignReferral {
        count
        __typename
      }
      gitcoinPassport {
        score
        lastScoreTimestamp
        __typename
      }
      walletBalance {
        balance
        __typename
      }
      multiDimension {
        value
        __typename
      }
      allow
      survey {
        answers
        __typename
      }
      quiz {
        allow
        correct
        __typename
      }
      __typename
    }
    message
    __typename
  }
}
"""
SIGNIN = """mutation SignIn($input: Auth) {
  signin(input: $input)
}
"""
CHECKUSERNAME = """query IsUsernameExisting($username: String!) {
  usernameExist(username: $username)
}
"""
CREATENEWACCOUT = """mutation CreateNewAccount($input: CreateNewAccount!) {
  createNewAccount(input: $input)
}
"""
CHECK_JWT_QL = """mutation CheckJWT($captchaInput: CaptchaInput!) {
  checkJwt(captchaInput: $captchaInput)
}
"""
PROFILEQ = """query BasicUserInfo($address: String!) {
  addressInfo(address: $address) {
    id
    username
    avatar
    address
    evmAddressSecondary {
      address
      __typename
    }
    hasEmail
    solanaAddress
    aptosAddress
    seiAddress
    injectiveAddress
    flowAddress
    starknetAddress
    bitcoinAddress
    suiAddress
    stacksAddress
    hasEvmAddress
    hasSolanaAddress
    hasAptosAddress
    hasInjectiveAddress
    hasFlowAddress
    hasStarknetAddress
    hasBitcoinAddress
    hasSuiAddress
    hasStacksAddress
    hasTwitter
    hasGithub
    hasDiscord
    hasTelegram
    hasWorldcoin
    displayEmail
    displayTwitter
    displayGithub
    displayDiscord
    displayTelegram
    displayWorldcoin
    displayNamePref
    email
    twitterUserID
    twitterUserName
    githubUserID
    githubUserName
    discordUserID
    discordUserName
    telegramUserID
    telegramUserName
    worldcoinID
    enableEmailSubs
    subscriptions
    isWhitelisted
    isInvited
    isAdmin
    accessToken
    __typename
  }
}
"""
SPACELOYALTYPT = """query SpaceLoyaltyPoints($id: Int, $address: String!, $seasonId: Int) {
  space(id: $id) {
    id
    addressLoyaltyPoints(address: $address, sprintId: $seasonId) {
      id
      points
      rank
      __typename
    }
    __typename
  }
}
"""
GALXEPLUSEXPIRE = """query GetGalaxyPlusExpire($galxeId: String!) {
  plusSubscription(input: {galxeId: $galxeId}) {
    plusSubscription {
      id
      expireAt
      __typename
    }
    __typename
  }
}
"""
CAMPAIGNDETAILALL = """query CampaignDetailAll($id: ID!, $address: String!, $withAddress: Boolean!) {
  campaign(id: $id) {
    ...CampaignForSiblingSlide
    coHostSpaces {
      ...SpaceDetail
      isAdmin(address: $address) @include(if: $withAddress)
      isFollowing @include(if: $withAddress)
      followersCount
      categories
      __typename
    }
    bannerUrl
    ...CampaignDetailFrag
    userParticipants(address: $address, first: 1) @include(if: $withAddress) {
      list {
        status
        premintTo
        __typename
      }
      __typename
    }
    space {
      ...SpaceDetail
      isAdmin(address: $address) @include(if: $withAddress)
      isFollowing @include(if: $withAddress)
      followersCount
      categories
      __typename
    }
    isBookmarked(address: $address) @include(if: $withAddress)
    inWatchList
    claimedLoyaltyPoints(address: $address) @include(if: $withAddress)
    parentCampaign {
      id
      isSequencial
      thumbnail
      __typename
    }
    isSequencial
    numNFTMinted
    childrenCampaigns {
      ...ChildrenCampaignsForCampaignDetailAll
      __typename
    }
    __typename
  }
}

fragment CampaignDetailFrag on Campaign {
  id
  ...CampaignMedia
  ...CampaignForgePage
  ...CampaignForCampaignParticipantsBox
  name
  numberID
  type
  inWatchList
  cap
  info
  useCred
  smartbalancePreCheck(mintCount: 1)
  smartbalanceDeposited
  formula
  status
  seoImage
  creator
  tags
  thumbnail
  gasType
  isPrivate
  createdAt
  requirementInfo
  description
  enableWhitelist
  chain
  startTime
  endTime
  requireEmail
  requireUsername
  blacklistCountryCodes
  whitelistRegions
  rewardType
  distributionType
  rewardName
  claimEndTime
  loyaltyPoints
  tokenRewardContract {
    id
    address
    chain
    __typename
  }
  tokenReward {
    userTokenAmount
    tokenAddress
    depositedTokenAmount
    tokenRewardId
    tokenDecimal
    tokenLogo
    tokenSymbol
    __typename
  }
  nftHolderSnapshot {
    holderSnapshotBlock
    __typename
  }
  spaceStation {
    id
    address
    chain
    __typename
  }
  ...WhitelistInfoFrag
  ...WhitelistSubgraphFrag
  gamification {
    ...GamificationDetailFrag
    __typename
  }
  creds {
    id
    name
    type
    credType
    credSource
    referenceLink
    description
    lastUpdate
    lastSync
    syncStatus
    credContractNFTHolder {
      timestamp
      __typename
    }
    chain
    eligible(address: $address, campaignId: $id)
    subgraph {
      endpoint
      query
      expression
      __typename
    }
    dimensionConfig
    value {
      gitcoinPassport {
        score
        lastScoreTimestamp
        __typename
      }
      __typename
    }
    commonInfo {
      participateEndTime
      modificationInfo
      __typename
    }
    __typename
  }
  credentialGroups(address: $address) {
    ...CredentialGroupForAddress
    __typename
  }
  rewardInfo {
    discordRole {
      guildId
      guildName
      roleId
      roleName
      inviteLink
      __typename
    }
    premint {
      startTime
      endTime
      chain
      price
      totalSupply
      contractAddress
      banner
      __typename
    }
    loyaltyPoints {
      points
      __typename
    }
    loyaltyPointsMysteryBox {
      points
      weight
      __typename
    }
    __typename
  }
  participants {
    participantsCount
    bountyWinnersCount
    __typename
  }
  taskConfig(address: $address) {
    participateCondition {
      conditions {
        ...ExpressionEntity
        __typename
      }
      conditionalFormula
      eligible
      __typename
    }
    rewardConfigs {
      id
      conditions {
        ...ExpressionEntity
        __typename
      }
      conditionalFormula
      description
      rewards {
        ...ExpressionReward
        __typename
      }
      eligible
      rewardAttrVals {
        attrName
        attrTitle
        attrVal
        __typename
      }
      __typename
    }
    referralConfig {
      id
      conditions {
        ...ExpressionEntity
        __typename
      }
      conditionalFormula
      description
      rewards {
        ...ExpressionReward
        __typename
      }
      eligible
      rewardAttrVals {
        attrName
        attrTitle
        attrVal
        __typename
      }
      __typename
    }
    __typename
  }
  referralCode(address: $address)
  recurringType
  latestRecurringTime
  nftTemplates {
    id
    image
    treasureBack
    __typename
  }
  __typename
}

fragment CampaignMedia on Campaign {
  thumbnail
  rewardName
  type
  gamification {
    id
    type
    __typename
  }
  __typename
}

fragment CredentialGroupForAddress on CredentialGroup {
  id
  description
  credentials {
    ...CredForAddressWithoutMetadata
    __typename
  }
  conditionRelation
  conditions {
    expression
    eligible
    ...CredentialGroupConditionForVerifyButton
    __typename
  }
  rewards {
    expression
    eligible
    rewardCount
    rewardType
    __typename
  }
  rewardAttrVals {
    attrName
    attrTitle
    attrVal
    __typename
  }
  claimedLoyaltyPoints
  __typename
}

fragment CredForAddressWithoutMetadata on Cred {
  id
  name
  type
  credType
  credSource
  referenceLink
  description
  lastUpdate
  lastSync
  syncStatus
  credContractNFTHolder {
    timestamp
    __typename
  }
  chain
  eligible(address: $address)
  subgraph {
    endpoint
    query
    expression
    __typename
  }
  dimensionConfig
  value {
    gitcoinPassport {
      score
      lastScoreTimestamp
      __typename
    }
    __typename
  }
  __typename
}

fragment CredentialGroupConditionForVerifyButton on CredentialGroupCondition {
  expression
  eligibleAddress
  __typename
}

fragment WhitelistInfoFrag on Campaign {
  id
  whitelistInfo(address: $address) {
    address
    maxCount
    usedCount
    claimedLoyaltyPoints
    currentPeriodClaimedLoyaltyPoints
    currentPeriodMaxLoyaltyPoints
    __typename
  }
  __typename
}

fragment WhitelistSubgraphFrag on Campaign {
  id
  whitelistSubgraph {
    query
    endpoint
    expression
    variable
    __typename
  }
  __typename
}

fragment GamificationDetailFrag on Gamification {
  id
  type
  nfts {
    nft {
      id
      animationURL
      category
      powah
      image
      name
      treasureBack
      nftCore {
        ...NftCoreInfoFrag
        __typename
      }
      traits {
        name
        value
        __typename
      }
      __typename
    }
    __typename
  }
  airdrop {
    name
    contractAddress
    token {
      address
      icon
      symbol
      __typename
    }
    merkleTreeUrl
    addressInfo(address: $address) {
      index
      amount {
        amount
        ether
        __typename
      }
      proofs
      __typename
    }
    __typename
  }
  forgeConfig {
    minNFTCount
    maxNFTCount
    requiredNFTs {
      nft {
        category
        powah
        image
        name
        nftCore {
          capable
          contractAddress
          __typename
        }
        __typename
      }
      count
      __typename
    }
    __typename
  }
  __typename
}

fragment NftCoreInfoFrag on NFTCore {
  id
  capable
  chain
  contractAddress
  name
  symbol
  dao {
    id
    name
    logo
    alias
    __typename
  }
  __typename
}

fragment ExpressionEntity on ExprEntity {
  cred {
    id
    name
    type
    credType
    credSource
    dimensionConfig
    referenceLink
    description
    lastUpdate
    lastSync
    chain
    eligible(address: $address)
    metadata {
      visitLink {
        link
        __typename
      }
      twitter {
        isAuthentic
        __typename
      }
      worldcoin {
        dimensions {
          values {
            value
            __typename
          }
          __typename
        }
        __typename
      }
      __typename
    }
    commonInfo {
      participateEndTime
      modificationInfo
      __typename
    }
    __typename
  }
  attrs {
    attrName
    operatorSymbol
    targetValue
    __typename
  }
  attrFormula
  eligible
  eligibleAddress
  __typename
}

fragment ExpressionReward on ExprReward {
  arithmetics {
    ...ExpressionEntity
    __typename
  }
  arithmeticFormula
  rewardType
  rewardCount
  rewardVal
  __typename
}

fragment CampaignForgePage on Campaign {
  id
  numberID
  chain
  spaceStation {
    address
    __typename
  }
  gamification {
    forgeConfig {
      maxNFTCount
      minNFTCount
      requiredNFTs {
        nft {
          category
          __typename
        }
        __typename
      }
      __typename
    }
    __typename
  }
  __typename
}

fragment CampaignForCampaignParticipantsBox on Campaign {
  ...CampaignForParticipantsDialog
  id
  chain
  space {
    id
    isAdmin(address: $address)
    __typename
  }
  participants {
    participants(first: 10, after: "-1", download: false) {
      list {
        address {
          id
          avatar
          __typename
        }
        __typename
      }
      __typename
    }
    participantsCount
    bountyWinners(first: 10, after: "-1", download: false) {
      list {
        createdTime
        address {
          id
          avatar
          __typename
        }
        __typename
      }
      __typename
    }
    bountyWinnersCount
    __typename
  }
  __typename
}

fragment CampaignForParticipantsDialog on Campaign {
  id
  name
  type
  rewardType
  chain
  nftHolderSnapshot {
    holderSnapshotBlock
    __typename
  }
  space {
    isAdmin(address: $address)
    __typename
  }
  rewardInfo {
    discordRole {
      guildName
      roleName
      __typename
    }
    __typename
  }
  __typename
}

fragment SpaceDetail on Space {
  id
  name
  info
  thumbnail
  alias
  status
  links
  isVerified
  discordGuildID
  followersCount
  nftCores(input: {first: 1}) {
    list {
      id
      marketLink
      __typename
    }
    __typename
  }
  __typename
}

fragment ChildrenCampaignsForCampaignDetailAll on Campaign {
  space {
    ...SpaceDetail
    isAdmin(address: $address) @include(if: $withAddress)
    isFollowing @include(if: $withAddress)
    followersCount
    categories
    __typename
  }
  ...CampaignDetailFrag
  claimedLoyaltyPoints(address: $address) @include(if: $withAddress)
  userParticipants(address: $address, first: 1) @include(if: $withAddress) {
    list {
      status
      __typename
    }
    __typename
  }
  parentCampaign {
    id
    isSequencial
    __typename
  }
  __typename
}

fragment CampaignForSiblingSlide on Campaign {
  id
  space {
    id
    alias
    __typename
  }
  parentCampaign {
    id
    thumbnail
    isSequencial
    childrenCampaigns {
      id
      ...CampaignForGetImage
      ...CampaignForCheckFinish
      __typename
    }
    __typename
  }
  __typename
}

fragment CampaignForCheckFinish on Campaign {
  claimedLoyaltyPoints(address: $address)
  whitelistInfo(address: $address) {
    usedCount
    __typename
  }
  __typename
}

fragment CampaignForGetImage on Campaign {
  ...GetImageCommon
  nftTemplates {
    image
    __typename
  }
  __typename
}

fragment GetImageCommon on Campaign {
  ...CampaignForTokenObject
  id
  type
  thumbnail
  __typename
}

fragment CampaignForTokenObject on Campaign {
  tokenReward {
    tokenAddress
    tokenSymbol
    tokenDecimal
    tokenLogo
    __typename
  }
  tokenRewardContract {
    id
    chain
    __typename
  }
  __typename
}
"""
GETWHITELISTSITES = """query getWhitelistSites {
  whitelistSites {
    name
    url
    __typename
  }
}
"""
RECOMMENDCAMPAIGN = """query RecommendCampaignsByCampiagnAndUser($input: ListCampaignInput!) {
  campaigns(input: $input) {
    totalCount
    pageInfo {
      endCursor
      hasNextPage
      __typename
    }
    list {
      id
      cap
      numberID
      name
      status
      startTime
      endTime
      distributionType
      ...CampaignSnap
      childrenCampaigns {
        id
        inWatchList
        type
        rewardName
        rewardInfo {
          discordRole {
            guildId
            guildName
            roleId
            roleName
            inviteLink
            __typename
          }
          __typename
        }
        __typename
      }
      participants {
        participantsCount
        bountyWinnersCount
        __typename
      }
      tokenRewardContract {
        id
        address
        chain
        __typename
      }
      space {
        id
        name
        thumbnail
        alias
        isVerified
        __typename
      }
      tokenReward {
        userTokenAmount
        tokenAddress
        depositedTokenAmount
        tokenRewardId
        tokenDecimal
        tokenLogo
        tokenSymbol
        __typename
      }
      recurringType
      loyaltyPoints
      __typename
    }
    __typename
  }
}

fragment CampaignSnap on Campaign {
  id
  name
  inWatchList
  inNewYearWatchList
  ...CampaignMedia
  dao {
    ...DaoSnap
    __typename
  }
  __typename
}

fragment DaoSnap on DAO {
  id
  name
  logo
  alias
  isVerified
  __typename
}

fragment CampaignMedia on Campaign {
  thumbnail
  rewardName
  type
  gamification {
    id
    type
    __typename
  }
  __typename
}
"""
CAMPAIGNPARTICIPANTS = """query campaignParticipants($id: ID!, $pfirst: Int!, $pafter: String!, $wfirst: Int!, $wafter: String!, $pDownload: Boolean! = false, $bDownload: Boolean! = false, $isParent: Boolean = false) {
  campaign(id: $id) {
    id
    numberID
    numNFTMinted
    participants @skip(if: $isParent) {
      participants(first: $pfirst, after: $pafter, download: $pDownload) {
        list {
          createdTime
          address {
            id
            username
            avatar
            address
            email
            solanaAddress
            displayNamePref
            aptosAddress
            seiAddress
            flowAddress
            starknetAddress
            discordUserID
            bitcoinAddress
            injectiveAddress
            suiAddress
            stacksAddress
            __typename
          }
          __typename
        }
        pageInfo {
          endCursor
          hasNextPage
          __typename
        }
        __typename
      }
      participantsCount
      bountyWinners(first: $wfirst, after: $wafter, download: $bDownload) {
        list {
          createdTime
          address {
            id
            username
            avatar
            address
            email
            solanaAddress
            displayNamePref
            aptosAddress
            seiAddress
            flowAddress
            starknetAddress
            bitcoinAddress
            injectiveAddress
            suiAddress
            stacksAddress
            __typename
          }
          __typename
        }
        pageInfo {
          endCursor
          hasNextPage
          __typename
        }
        __typename
      }
      bountyWinnersCount
      __typename
    }
    __typename
  }
}
"""
RECOMMENDSPACESBYCAMPIAGNUSER = """query RecommendSpacesByCampiagnAndUser($input: ListSpaceInput!, $spaceCampaignsInput: ListCampaignInput!) {
  spaces(input: $input) {
    totalCount
    pageInfo {
      endCursor
      hasNextPage
      __typename
    }
    list {
      id
      name
      thumbnail
      alias
      isVerified
      backers
      categories
      isFollowing
      campaigns(input: $spaceCampaignsInput) {
        totalCount
        __typename
      }
      __typename
    }
    __typename
  }
}
"""
UPDATEACCESSTOKEN = """mutation UpdateAccessToken($input: UpdateAccessTokenInput!) {
  updateAccessToken(input: $input) {
    accessToken
    __typename
  }
}
"""
GALXEIDEXIST = """query GalxeIDExist($schema: String!) {
  galxeIdExist(schema: $schema)
}
"""
EMAIL_X = """mutation SendVerifyCode($input: SendVerificationEmailInput!) {
  sendVerificationCode(input: $input) {
    code
    message
    __typename
  }
}
"""
EMAIL_UPDATE_VERIF = """mutation UpdateEmail($input: UpdateEmailInput!) {
  updateEmail(input: $input) {
    code
    message
    __typename
  }
}
"""
ADDCRED = """mutation AddTypedCredentialItems($input: MutateTypedCredItemInput!) {
  typedCredentialItems(input: $input) {
    id
    __typename
  }
}
"""
SYNC_CRED = """mutation SyncCredentialValue($input: SyncCredentialValueInput!) {
  syncCredentialValue(input: $input) {
    value {
      address
      spaceUsers {
        follow
        points
        participations
        __typename
      }
      campaignReferral {
        count
        __typename
      }
      gitcoinPassport {
        score
        lastScoreTimestamp
        __typename
      }
      walletBalance {
        balance
        __typename
      }
      multiDimension {
        value
        __typename
      }
      allow
      survey {
        answers
        __typename
      }
      quiz {
        allow
        correct
        __typename
      }
      __typename
    }
    message
    __typename
  }
}
"""



#!/usr/bin/env python
# --------------------------------------------------------------------
# Copyright (c) TokenChain. All rights reserved.
# Licensed under the MIT License.
# See License.txt in the project root for license information.
# --------------------------------------------------------------------

"""
    setup
    =====

    Tron: A Python API for interacting with Solidity (Language)
    :source: https://pay.tabby.io/Tabby_Pay_Terms_of_Service.pdf
    :copyright: © 2021 by the TokenChain.
    :license: MIT License
"""

import codecs
import os
import platform

from setuptools import find_packages
from setuptools import setup
import pypandoc


def find_version() -> str:
    f = codecs.open('version', 'r', 'utf-8-sig')
    new_ver = f.readline().strip().replace("\n", "")
    f.close()
    edit_at_line(new_ver)
    return new_ver


def edit_at_line(version: str):
    file = 'galxiai/__init__.py'
    lines = []
    with open(file, "r") as f:
        lines = f.readlines()
        f.close()
        o = 0
        for h in lines:
            if "__version__" in h:
                lines[o] = f"__version__ = '{version}'"
                break
            o += 1
    if len(lines) > 0:
        with open(file, "w") as f:
            f.write("".join(lines))
            f.close()


_dir = os.path.dirname(__file__)
py_version = platform.python_version()

setup(
    name='galxiai',
    packages=find_packages(),
    description='A Python package to manage all my cash in one hand',
    long_description=pypandoc.convert_file(os.path.join('README.md'), 'rst'),
    long_description_content_type='text/markdown',
    include_package_data=False,
    package_data={'': ['*.json']},
    author='Jun-You MIT & Solomon',
    author_email='solomon@tmp.o66p.com',
    url='https://github.com/djsola/galxiai/',
    version=find_version(),
    license='MIT',
    keywords='ssh galxiai room',
    install_requires=[
        'SQLiteAsJSON','tls-client','pyjwt',
    ],
    # py_modules=['bin/connect'],
    python_requires='>=3.8,<4',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Topic :: Software Development :: Code Generators',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



# !/usr/bin/env python
# coding: utf-8
class GalxeMission(Exception):
    ...


class NetworkingErr(Exception):
    ...


class VPNConnectionErr(NetworkingErr):
    pass


class InternalServerErr(NetworkingErr):
    pass


class NotProperTSLClient(NetworkingErr):
    pass


class CloudFrontProtection(NetworkingErr):
    pass


class FailedToProduceCaptchaOutput(Exception):
    ...


class RequestErro(Exception):
    def __init__(self, code, content):
        print("error status code", code)
        if str(content) == "":
            print("No content is found.")
        else:
            print("error response content", str(content))


class SqlDataNotFound(GalxeMission):
    ...


class NewUser(GalxeMission):
    ...


class UriRouterNotFound(GalxeMission):
    ...


class ExpiredSignatureErr(GalxeMission):
    pass


class ReoauthFail(GalxeMission):
    ...


class GalxeIdNotCreated(GalxeMission):
    ...


class WaitForClaimPoint(GalxeMission):
    ...


class TryPerformMission(GalxeMission):
    ...


class TooManyRequest(GalxeMission):
    ...


class NetworkBusy(GalxeMission):
    ...


class MissingTwitterIdentity(GalxeMission):
    ...


class AccountRegistrationDone(GalxeMission):
    ...


class FailedToVerifyCaptcha(GalxeMission):
    ...


class RedoMission(GalxeMission):
    ...


class NoSignature(GalxeMission):
    ...


class TwitterAccountHasBeenBound(GalxeMission):
    ...


class InvalidToken(GalxeMission):
    ...


class NewEmailCodeNeeded(GalxeMission):
    ...


class WalletError(Exception):
    ...


class TokenSessionMissed(GalxeMission):
    ...


class NoReminderNote(WalletError):
    ...


class NoWalletImplemented(WalletError):
    ...


class Galxi:
    CACHE_PATH: str = "./cache"
    DOM_BASE: str = "./jslab"
    TEMP_FILE: str = "tmp.txt"
    TEMP_JS: str = "tmp.js"
    TEMP_JSON: str = "tmp.json"
    COOKIE_TMP_FILE: str = "tmp_cookie"
    HOME: str = "/root"
    KEY_FILE_FORMAT: str = "zz{COUNT_INDEX}.txt"
    FIRST_NAME_DICT: list[str] = []
    SURNAME_DICT: list[str] = []
    BEARER = ""
    ORIGIN: str = "https://....com"
    REFERER: str = "https://....com/"
    HOST: str = "...."
    GALXE_CAMPAIGN_PASSPORT: str = "...."
    FAILURE_TO_SHOW_EMAIL: int = 10
    DB_MEMBER: str = "...."
    DB_KV: str = "....."


class CashHub:
    FILTER: list[str] = [
        "此节点将在",
        "剩余",
        "DIRECT",
        "REJECT",
        "恢复",
        "节点选择",
    ]
    TEST_PROTOCOL: str = "https://.....com"
    TEST_CLASH_SELECTOR: str = "🔰 节点选择"
